from whacc import utils, image_tools, transfer_learning
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras
from sklearn.utils import class_weight
import time
from pathlib import Path
import os
import copy
import numpy as np
from tensorflow.keras import applications

"""

for running this with the real dataset I will have to make subset of the 10% dataset using 
    split_h5_loop_segments
i might also have to run 
    make_sure_frame_nums_exist
on all the files first  
to make sure frame_nums is in the keys 


here at the beginning I need to run the combine and split the touches. make sure they are all the same. I want to make the 
aug and split it before I do anything that is the OG that I upload. 
"""


def build_model(info_dict, labels, model_name_str, base_learning_rate=0.00001, dropout_val=None, class_numbers=None,
                IMG_SIZE=96):
    if class_numbers is None:
        class_numbers = np.unique(labels)
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.AUC(name='auc'),
                   keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5)]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
    class_weights = {i: rebalance[i] for i in class_numbers}
    wrap_vars_list = ['class_numbers',
                      'num_classes',
                      'base_learning_rate',
                      'model_name_str',
                      'IMG_SIZE',
                      'dropout_val',
                      'class_weights']
    for k in wrap_vars_list:
        info_dict[k] = eval(k)
    return model, class_weights, info_dict


def label_naming_shorthand_dict(name_key=None):
    label_naming_shorthand_dict = {
        '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
        '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
        '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
        '[0, 1]- (no touch, touch)': 'regular',
        '[0, 1]- (not offset, offset)': 'only_offset',
        '[0, 1]- (not onset, onset)': 'only_onset',
        '[0, 1, 2, 3]- (no touch, touch, one after onset, offset)': 'overlap_whisker_on-off'}
    if name_key is None:
        return label_naming_shorthand_dict
    else:
        return label_naming_shorthand_dict[name_key]


def info_dict_wrapper(info_dict, local_dict):
    for k in local_dict.keys():
        info_dict[k] = local_dict[k]
    return info_dict


def make_initial_folder(all_models_directory, unique_h5_train_val_dir):
    single_frame_dir = all_models_directory + os.sep + unique_h5_train_val_dir + os.sep + 'data' + os.sep + 'single_frame'
    Path(single_frame_dir).mkdir(parents=True, exist_ok=True)
    return locals()
    # return single_frame_dir


def get_automated_model_info(BASE_H5, image_source_h5_directory_ending, test_data_dir):
    todays_version = time.strftime("%Y_%m_%d_%H_%S", time.gmtime())
    a = os.sep
    base_data_dir = BASE_H5 + a + "data" + a
    base_dir_all_h5s = BASE_H5 + a + 'data' + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    image_h5_list = utils.get_h5s(data_dir)
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    labels_dir = base_data_dir + a + "ALT_LABELS" + a
    print('\nFOR LABELS,0 is train set, 1 is val set')
    label_h5_list = utils.get_h5s(labels_dir)
    print('\nSelect from the following label structures...')
    label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)

    h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()


def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
    return locals()


# get list of pre trained models to choose from
def get_keras_model_names():
    names_, types_ = utils.get_class_info(applications, return_name_and_type=True)
    model_names = np.asarray(names_)['function' == np.asarray(types_)]
    utils.print_list_with_inds(model_names)
    return model_names


def make_model_save_directory(info_dict, make_folder=True):
    naming_list = ['model_name_str', 'image_source_h5_directory_ending', 'label_key_shorthand', 'todays_version']
    model_save_dir: str = copy.deepcopy(info_dict['BASE_H5'])
    for k in naming_list:
        model_save_dir += os.sep + info_dict[k] + os.sep
    info_dict['model_save_dir'] = model_save_dir
    info_dict['model_save_dir_checkpoints'] = model_save_dir + os.sep + 'checkpoints'
    if make_folder:
        Path(info_dict['model_save_dir_checkpoints']).mkdir(parents=True, exist_ok=True)
    return model_save_dir


def basic_callbacks(save_checkpoint_filepath, monitor='val_loss', patience=10,
                    save_best_only=False, save_weights_only=True, save_freq=10):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, patience=patience))
    add_path_name = "{loss:.8f}_{epoch:04d}_cp.hdf5"

    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath + os.sep + add_path_name,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq))
    return callbacks



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$      make/setup data     $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

info_dict = dict()
all_models_directory = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/"  # DETERMINES location for all model you will run
# info_dict['test_data_dir'] =
test_data_dir =        '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/'  # DETERMINES location for test data (folder determined by the "image_source_h5_directory_ending" variable)

# '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data'

unique_h5_train_val_dir = 'small_h5s'  # DETERMINES the name of the folder where each type of data is stored
unique_h5_train_val_dir = 'regular_80_border'
image_source_h5_directory_ending = "/regular/"  # DETERMINES THE IMAGE SOURCE
make_labels_and_h5s = True  # DETERMINES if you re run the data transformation form the single frames into the regular, 3lag and diff_3lag

model_names = get_keras_model_names()  # get and print model names to choose from
model_name_str = 'InceptionV3'  # DETERMINES model base
base_learning_rate = 0.00001  # DETERMINES rate of change for each epoch step
dropout_val = 0.25  # DETERMINES percentage of dropout for training data
patience = 10  # DETERMINES early stopping
save_freq = 2  # DETERMINES how often it saves the checkpoints
epochs = 5000  # DETERMINES how many epochs the model trains for if early stopping is never triggered
batch_size = 1000  # DETERMINES number of images per batch

label_key_name_list = label_naming_shorthand_dict()  # get a list of label key names... they are really long to be specific
utils.print_list_with_inds(label_key_name_list, )  # print them, then below use their index to choose them
label_key_name = list(label_key_name_list.keys())[
    0]  # DETERMINES THE LABEL SOURCE choose the ind based on the print out

local_dict = make_initial_folder(all_models_directory,
                                 unique_h5_train_val_dir)  # make data single frame directory and get that directory
info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

BASE_H5 = info_dict['all_models_directory'] + os.sep + info_dict[
    'unique_h5_train_val_dir']  # directory for all the data for a certain type og images(lag or regular etc)
if make_labels_and_h5s:  # make all the h5 files with lag and subtraction and regular
    base_dir_all_h5s = BASE_H5 + os.sep + 'data' + os.sep + 'single_frame' + os.sep
    utils.make_all_H5_types(
        base_dir_all_h5s)  # auto generate all the h5 types using a single set of flat (no color) image H5
    utils.make_alt_labels_h5s(base_dir_all_h5s)  # auto generate the different types of labels
local_dict = get_automated_model_info(BASE_H5, image_source_h5_directory_ending,
                                      test_data_dir)  # basic data like directories and train and val set (automated form the directory)
info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
local_dict = copy_over_new_labels(label_key_name, info_dict['image_h5_list'] + [info_dict['h5_test']],
                                  info_dict['label_h5_list'] + [info_dict[
                                                                    'h5_test_labels']])  # copy specific labels to the H5 of interest (this is all after the conversion from single frame)
info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$      make model     $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# get all the labels
train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']])
val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']])
labels = np.concatenate((train_labels, val_labels))

model, class_weights, info_dict = build_model(info_dict, labels, model_name_str, base_learning_rate=base_learning_rate,
                                              dropout_val=dropout_val)
model_save_dir = make_model_save_directory(
    info_dict)  # make a unique folder using standard folder struct ending in a unique date/time folder

model, class_weights, info_dict = build_model(info_dict, labels, model_name_str, base_learning_rate=base_learning_rate,
                                              dropout_val=dropout_val)

callbacks = basic_callbacks(info_dict['model_save_dir_checkpoints'], monitor='val_loss', patience=patience,
                            save_best_only=False, save_weights_only=True, save_freq=save_freq)

train_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_train']])
val_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_val']])
test_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_test']])

plot_callback = transfer_learning.save_and_plot_history(test_gen, key_name='labels', plot_metric_inds=[0])
# transfer_learning.foo_save_and_plot(training_info, plot_callback, "/content/gdrive/My Drive/Colab data/SAMSON/model_tests/")
callbacks.append(plot_callback)
history = model.fit(train_gen,
                    epochs=epochs,
                    validation_data=val_gen,
                    callbacks=callbacks,
                    class_weight=class_weights)


def pack_training_info(save_and_plot_history_var):
    training_info = dict()
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    # for k in training_info.keys():
    #     if 'numpy.ndarray' in str(type(training_info[k])):
    #         training_info[k] = training_info[k].tolist()
    return training_info
training_info = pack_training_info(plot_callback)
utils.get_class_info(plot_callback)









"""
make test set locations and add to info_dict

add my custom callback with test set


"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""



# # #1 get a single folder named "single_frame" and put the test val and train sets in there, each of these folders wiwill be a different
# # # combination of the of regular vs augmented images ... should i use only augmented images for the aug set? NO jsut add teh partial aug set
# # base_dir_all_h5s = BASE_H5 + '/data/single_frame/' # MUST but a separate folder
# # # with final 'single frame' H5s. i.e. split and aug and put into train val and test sets (ONLY THESE 3!!)
# # # base_dir = str(Path(base_dir_all_h5s).parent.absolute())
# make_labels_and_h5s = False
# if make_labels_and_h5s: # make all the h5 files with lag and subtraction and regular
#     utils.make_all_H5_types(base_dir_all_h5s) # auto generate all the h5 types using a single set of flat (no color) image H5
#     utils.make_alt_labels_h5s(base_dir_all_h5s) # auto generate the different types of labels
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

todays_version = time.strftime("%Y_%m_%d_%H_%S", time.gmtime())

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  DETERMINES THE IMAGE SOURCE  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
base_data_dir = BASE_H5+ "/data/"
image_source_h5_directory_ending = "/regular/" # DETERMINES THE IMAGE SOURCE
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
data_dir = base_data_dir + image_source_h5_directory_ending
image_h5_list = utils.get_h5s(data_dir)
h5_test = image_h5_list[0]
h5_train = image_h5_list[1]
h5_val =image_h5_list[2]
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  DETERMINES THE LABEL SOURCE  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
labels_dir = base_data_dir + "/ALT_LABELS/"
label_h5_list = utils.get_h5s(labels_dir)
label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
label_key_name = label_key_name_list[0] # DETERMINES THE LABEL SOURCE
label_key_shorthand = label_naming_shorthand_dict(label_key_name)

for img_src, lab_src in zip(image_h5_list, label_h5_list):
    utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')


# make model for each model type
"""$$$$$$$$$$$  make a model maker for each of these model types with their respective characteristics  $$$$$$$$$$$$$$"""
"""
break down into modules

make_model
class weights module
num classes auto
dropout input
"""

model_names = get_keras_model_names()

train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']])
val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']])
labels = np.concatenate((train_labels, val_labels))

model_name_str = 'InceptionV3'# DETERMINES
base_learning_rate = 0.00001# DETERMINES
dropout_val = 0.25 # DETERMINES




model, class_weights, info_dict = build_model(info_dict, labels, model_name_str, base_learning_rate = base_learning_rate, dropout_val = dropout_val)


def basic_callbacks(save_checkpoint_filepath, monitor = 'val_loss', patience = 10,
                    save_best_only = False, save_weights_only = True, save_freq = 10):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor = monitor,patience = patience)])

    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq))

"""

add the
.ckpt
to save all the checkpoints

and use .pb to save the whole model at the end

check the save number using the
filepath = ".\checkpoints\cp-{epoch:04d}.hdf5"


"""

"""
sadfasdf
"""
base_learning_rate = 0.00001

class_numbers = [0, 1, 2, 3] # auto use unique(labels)
num_classes = len(class_numbers)
IMG_SIZE = 96 # All images will be resized to 96x96. This is the size of MobileNetV2 input sizes


IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
base_model = keras.applications.InceptionV3(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
base_model.trainable = True
x = base_model.output
x = keras.layers.GlobalAveragePooling2D()(x)# global spatial average pooling layer
x = Dense(512, activation='relu')(x)#  fully-connected layer
x = Dropout(0.5)(x)
predictions = Dense(num_classes, activation='softmax')(x)# fully connected output/classification layer

base_learning_rate = 0.00001
model = Model(inputs=base_model.input, outputs=predictions)
METRICS = [
      keras.metrics.SparseCategoricalAccuracy(name = 'acc')]
model.compile(optimizer=keras.optimizers.Adam(learning_rate=base_learning_rate),
              loss=keras.losses.SparseCategoricalCrossentropy(from_logits=False), # default from_logits=False
              metrics=METRICS)



utils.print_h5_keys(f)
import h5py
with h5py.File(f, 'r')



rebal_labels = image_tools.get_h5_key_and_concatenate([h5_train, h5_val], 'labels')
start = time.time()

# Fit model with a couple parameters
EPOCHS = 1

# Class imbalance weighting
rebalance = class_weight.compute_class_weight('balanced',
                                  [class_numbers], four_class_labels.flatten())
class_weights = {i : rebalance[i] for i in range(4)}

# Early stopping
callbacks = [keras.callbacks.EarlyStopping (monitor = 'val_loss',patience = 10)]



history = model.fit(my_training_batch_generator, epochs=EPOCHS,
              validation_data= my_validation_batch_generator,
              callbacks = callbamodel_name_strcks,
              class_weight = class_weights)




total_seconds = time.time() - start
print('total run time :' + str(round(total_seconds/60)), ' minutes')

todays_version = time.strftime("%Y%m%d%s", time.gmtime())

SAVE_NAME = 'lagged_4_class_inceptionV3'

end_dir = model_save_dir + '/' + SAVE_NAME + todays_version +'.ckpt'
model.save_weights(end_dir)
end_dir = model_save_dir + '/' + SAVE_NAME + todays_version +'.ckpt'
model.save(end_dir)
# # TO SAVE HISTORY INFO
# hist_save_name = [model_save_dir+ 'HISTORY_'+ 1 + todays_version+'.json']
# with open(hist_save_name[0], 'w') as f:
#     json.dump(history.history, f)

tmp1 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_single_frames/data/single_frame/'
utils.make_all_H5_types(tmp1) # auto generate all the h5 types using a single set of flat (no color) image H5
utils.make_alt_labels_h5s(tmp1) # auto generate the different types of labels


tmp1 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/10_percent_holy_set/data/single_frame/'
utils.make_all_H5_types(tmp1) # auto generate all the h5 types using a single set of flat (no color) image H5
utils.make_alt_labels_h5s(tmp1) # auto generate the different types of labels



# # # # # # # # # # # import h5py
# # # # # # # # # # # import matplotlib.pyplot as plt
# # # # # # # # # # # from whacc import image_tools
# # # # # # # # # # # import os
# # # # # # # # # # # from whacc import utils
# # # # # # # # # # #
# # # # # # # # # # # for h5_file in utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/OLD_2'):
# # # # # # # # # # #     with h5py.File(h5_file, 'r') as h:
# # # # # # # # # # #         plt.figure(figsize=[20, 10])
# # # # # # # # # # #         plt.imshow(image_tools.img_unstacker(h['images'][:],40))
# # # # # # # # # # #         plt.title(h5_file.split('/')[-1])
# # # # # # # # # # #         plt.savefig(h5_file[:-3] + "_fig_plot.png")
# # # # # # # # # # #         plt.close()
# # # # # # # # # # #
# # # # # # # # # # #
# # # # # # # # # # # for k in utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/OLD_2'):
# # # # # # # # # # #     with h5py.File(k, 'r') as h:
# # # # # # # # # # #         k2 = k[:-3]+'_TMP.h5'
# # # # # # # # # # #         with h5py.File(k2, 'w') as h2:
# # # # # # # # # # #             h2.create_dataset('all_inds', data=h['all_inds'][50:])
# # # # # # # # # # #             h2.create_dataset('images', data=h['images'][50:])
# # # # # # # # # # #             h2.create_dataset('in_range', data=h['in_range'][50:])
# # # # # # # # # # #             h2.create_dataset('labels', data=h['labels'][50:])
# # # # # # # # # # #             h2.create_dataset('retrain_H5_info', data=h['retrain_H5_info'][:])
# # # # # # # # # # #     os.remove(k)
# # # # # # # # # # #     os.rename(k2, k)
# # # # # # # # # # #
# # # # # # # # # # #
# # # # # # # # # # # import h5py
# # # # # # # # # # # import matplotlib.pyplot as plt
# # # # # # # # # # # from whacc import image_tools
# # # # # # # # # # #
# # # # # # # # # # # h5_file= '/Users/phil/Downloads/AH1159X18012021xS404_subset (1).h5'
# # # # # # # # # # # with h5py.File(h5_file, 'r') as h:
# # # # # # # # # # #     plt.figure(figsize=[20, 10])
# # # # # # # # # # #     plt.imshow(image_tools.img_unstacker(h['images'][:],10))
# # # # # # # # # # #     plt.title(h5_file.split('/')[-1])
# # # # # # # # # # #     print(len(h['labels'][:]))
# # # # # # # # # # #
# # # # # # # # # # #
# # # # # # # # # # #
# # # # # # # # # # #
# # # # # # # # # #
# # # # # # # # # # import numpy as np
# # # # # # # # # # import whacc
# # # # # # # # # # from whacc import analysis
# # # # # # # # # # import matplotlib.pyplot as plt
# # # # # # # # # #
# # # # # # # # # #
# # # # # # # # # # a = analysis.pole_plot(
# # # # # # # # # #     '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/master_train_test1.h5',
# # # # # # # # # #     pred_val = [0,0,0,0,0,0,0,.2,.4,.5,.6,.7,.8,.8,.6,.4,.2,.1,0,0,0,0],
# # # # # # # # # #     true_val = [0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
# # # # # # # # # #     len_plot = 10)
# # # # # # # # # #
# # # # # # # # # # a.plot_it()
# # # # # # # # # #
# # # # # # # # # #
# # # # # # # # # f1 = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/delete_after_oct_2021/AH0000x000000.h5"
# # # # # # # # # import h5py
# # # # # # # # #
# # # # # # # # # with h5py.File(f1, 'r') as h:
# # # # # # # # #     k = 'file_name_nums'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'full_file_names'
# # # # # # # # #     print(k, '   ', (h[k].shape))
# # # # # # # # #     k = 'images'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'in_range'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'labels'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'locations_x_y'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'max_val_stack'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'multiplier'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #     k = 'trial_nums_and_frame_nums'
# # # # # # # # #     print(k, '   ', h[k].shape)
# # # # # # # # #
# # # # # # # # # with h5py.File(save_directory + file_name, 'r+') as hf:  # with -> auto close in case of failure
# # # # # # # # #     hf.create_dataset("asdfasdf", data=loc_stack_all2)
# # # # # # # # #     for i, k in enumerate(loc_stack_all):
# # # # # # # # #         print(i)
# # # # # # # # #         hf.create_dataset("aaa"+str(i), data=k)
# # # # # # # # #
# # # # # # # # # loc_stack_all2 = loc_stack_all.copy()
# # # # # # # # #
# # # # # # # # #
# # # # # # # # #
# # # # # # # # #
# # # # # # # # # for i, k in enumerate(loc_stack_all):
# # # # # # # # #     if not k.shape==(4000, 2):
# # # # # # # # #         print(i)
# # # # # # # # #     # for kk in k.flatten():
# # # # # # # # #     #     print(kk.shape)
# # # # # # # # #     #         # if not kk.dtype=='int64':
# # # # # # # # #     #         #     print('booooo')
# # # # # # # # #     #
# # # # # # # # #     #
# # # # # # # #
# # # # # # # #
# # # # # # # # # import h5py
# # # # # # # # # import numpy as np
# # # # # # # # # import matplotlib.pyplot as plt
# # # # # # # # # cnt = 0
# # # # # # # # # f = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
# # # # # # # # # with h5py.File(f, 'r') as h:
# # # # # # # # #     print(len(h['labels']))
# # # # # # # # #     for k in range(0, 1163998, 20000):
# # # # # # # # #         plt.figure()
# # # # # # # # #         plt.imshow(h['images'][k])
# # # # # # # # #         cnt+=1
# # # # # # # # #         # if cnt>=20 :
# # # # # # # # #         #     asdfasdf
# # # # # # # #
# # # # # # # # from whacc import utils
# # # # # # # # from whacc import image_tools
# # # # # # # # import copy
# # # # # # # # import h5py
# # # # # # # # f = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5'
# # # # # # # # f2 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5'
# # # # # # # #
# # # # # # # # h5c = image_tools.h5_iterative_creator(f2,
# # # # # # # #                  overwrite_if_file_exists=False,
# # # # # # # #                  max_img_height=61,
# # # # # # # #                  max_img_width=61,
# # # # # # # #                  close_and_open_on_each_iteration=True,
# # # # # # # #                  color_channel=False)
# # # # # # # # with h5py.File(f, 'r') as h:
# # # # # # # #     print(len(h['labels'][:]))
# # # # # # # #     h_cont = copy.deepcopy(h['labels'][:])
# # # # # # # # utils.create_master_dataset(h5c, [f], [h_cont], borders=80, max_pack_val=100)
# # # # # # # #
# # # # # # # #
# # # # # # # # ####
# # # # # # # #
# # # # # # # #
# # # # # # # # f = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5"
# # # # # # # # with h5py.File(f, 'r') as h:
# # # # # # # #     utils.print_list_with_inds(h.keys())
# # # # # # # #     tmp1 = copy.deepcopy(h['frame_nums'][:])
# # # # # # # #
# # # # # # # #
# # # # # # # # """
# # # # # # # # add teh frame and trail number to add teh frame num info to each with the same name!!!!
# # # # # # # # """
# # # # # # # #
# # # # # # # # h5file = "/Users/phil/Downloads/untitled folder 2/AH0000x000000_small_tester.h5"
# # # # # # # # def make_sure_frame_nums_exist(h5file):
# # # # # # # #     with h5py.File(h5file, 'r+') as h:
# # # # # # # #         key_list = list(h.keys())
# # # # # # # #         if 'frame_nums' in key_list:
# # # # # # # #             print("""'frame_nums' already in the key list""")
# # # # # # # #             return None
# # # # # # # #         assert 'trial_nums_and_frame_nums' in key_list, """key 'trial_nums_and_frame_nums' must be in the provided h5 this is the only reason program exists"""
# # # # # # # #         frame_nums = h['trial_nums_and_frame_nums'][1, :]
# # # # # # # #         h.create_dataset('frame_nums', shape=np.shape(frame_nums), data=frame_nums)
# # # # # # # # make_sure_frame_nums_exist(h5file)
# # # # # # # #
# # # # # # # # frame_num_array_list = get_h5_key_and_dont_concatenate(h5_to_split_list, key_name='trial_nums_and_frame_nums')
# # # # # # # # frame_num_array_list = list(frame_num_array_list[0][-1])
# # # # # # # # frame_num_array_list = [frame_num_array_list]
# # # # # # # # bd = '/Users/phil/Downloads/untitled folder 2/'
# # # # # # # # split_h5_loop_segments(h5_to_split_list, [1, 3], [bd+'TRASH', bd+'TRASH2'], frame_num_array_list, chunk_size=10000, add_numbers_to_name=False,
# # # # # # # #              disable_TQDM=False, set_seed = None)
# # # # # # # #
# # # # # # # #
# # # # # # # # h5file = '/Users/phil/Downloads/untitled folder 2/TRASH2.h5'
# # # # # # # # utils.print_h5_keys(h5file)
# # # # # # # # with h5py.File(h5file, 'r') as h:
# # # # # # # #     print(len(h['labels'][:]))
# # # # # # # #     print(h['frame_nums'][:])
# # # # # # # #
# # # # # # #
# # # # # # #
# # # # # # from whacc import image_tools, utils
# # # # # # h5_to_split_list = "/Users/phil/Downloads/untitled folder 2/AH0000x000000_small_tester.h5"
# # # # # # h5_to_split_list = [h5_to_split_list]
# # # # # # utils.print_h5_keys(h5_to_split_list[0])
# # # # # # bd = '/Users/phil/Downloads/untitled folder 2/'
# # # # # # image_tools.split_h5_loop_segments(h5_to_split_list, [1, 3], [bd+'TRASH', bd+'TRASH2'], chunk_size=10000, add_numbers_to_name=False,
# # # # # #              disable_TQDM=False, set_seed = None)
# # # # # #
# # # # # #
# # # # # # from whacc import utils
# # # # # # from whacc import image_tools
# # # # # # from keras.preprocessing.image import ImageDataGenerator
# # # # # # import h5py
# # # # # # import matplotlib.pyplot as plt
# # # # # # from tqdm import tqdm
# # # # # # from imgaug import augmenters as iaa  # optional program to further augment data
# # # # # # h5_subset_file = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/AH0000x000000_5border_single_frame_onset_offset_for_aug_3lag_remove_2_edges.h5'
# # # # # # for h5_subset_file in utils.get_files('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/', '*remove_2_edges.h5'):
# # # # # #     datagen = ImageDataGenerator(rotation_range=360,  #
# # # # # #                                  width_shift_range=.1,  #
# # # # # #                                  height_shift_range=.1,  #
# # # # # #                                  shear_range=.00,  #
# # # # # #                                  zoom_range=.25,
# # # # # #                                  brightness_range=[0.2, 1.2])  #
# # # # # #
# # # # # #     with h5py.File(h5_subset_file, 'r') as hf:
# # # # # #         test_img = hf['images'][2]  # grab a single image
# # # # # #     num_aug = 20
# # # # # #
# # # # # #     aug_ims, _ = image_tools.augment_helper(datagen, num_aug, 0 ,test_img, -1)  # make 99 of the augmented images and output 1 of th original
# # # # # #     # '-1' is the label associate ith the image, we dont care becasue we just want the iamge so it doenst matter what it is here
# # # # # #
# # # # # #     # plt.figure(figsize=[5, 5])
# # # # # #     # plt.imshow(image_tools.img_unstacker(aug_ims, 10), )
# # # # # #     # plt.show()
# # # # # #
# # # # # #     gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
# # # # # #     aug_ims_2 = gaussian_noise.augment_images(aug_ims)
# # # # # #     # plt.figure(figsize=[10, 10])
# # # # # #     # plt.imshow(image_tools.img_unstacker(aug_ims_2, 10))
# # # # # #     # plt.show()
# # # # # #
# # # # # #     num_aug = 1
# # # # # #     for ii in range(10):
# # # # # #         # once we are happy with our augmentation process we can make an augmented H5 file using class
# # # # # #         new_H5_file = h5_subset_file.split('.')[0] + '_AUG_' + str(ii) + '.h5' # create new file name based on teh original H5 name
# # # # # #         h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
# # # # # #                                                      close_and_open_on_each_iteration=True, color_channel=True)
# # # # # #         utils.get_class_info(h5creator)
# # # # # #         with h5py.File(h5_subset_file, 'r') as hf:
# # # # # #             for image, label in zip(tqdm(hf['images'][:]), hf['labels'][:]):
# # # # # #                 aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
# # # # # #                 aug_img_stack = gaussian_noise.augment_images(aug_img_stack) # optional
# # # # # #                 h5creator.add_to_h5(aug_img_stack[:, :, :, :], labels_stack)
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # # h5 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_0.h5'
# # # # # # a = analysis.pole_plot(h5)
# # # # # # a.next()
# # # # # # h5 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_1.h5'
# # # # # # b = analysis.pole_plot(h5)
# # # # # # b.next()
# # # # # #
# # # # # #
# # # # # #
# # # # # # from whacc import utils
# # # # # # all_h5s = utils.get_h5s('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/finished_contacts/')
# # # # # # all_h5s_imgs = utils.get_h5s('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/H5_data/')
# # # # # # h_cont = utils._get_human_contacts_(all_h5s)
# # # # # # h5c = image_tools.h5_iterative_creator('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/holy_set_80_border_single_frame.h5',
# # # # # #                                        overwrite_if_file_exists = True,
# # # # # #                                        color_channel = False)
# # # # # # utils.create_master_dataset(h5c, all_h5s_imgs, h_cont, borders = 80, max_pack_val = 100)
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # # #
# # # # # # # from whacc import image_tools
# # # # # # #
# # # # # # # # bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/'
# # # # # # # # h5_to_split_list = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5']
# # # # # # # # image_tools.split_h5_loop_segments(h5_to_split_list, [7, 3], [bd+'train', bd+'val'], chunk_size=1000, add_numbers_to_name=False,
# # # # # # # #              disable_TQDM=False, set_seed = 0, color_channel=False)
# # # # # # # #
# # # # # # #
# # # # # # #
# # # # # # # bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9_test/'
# # # # # # # h5_to_split_list = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoC2uratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5',
# # # # # # #                     '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_9.h5']
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_8.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_7.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_6.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_5.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_4.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_3.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_2.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_1.h5',
# # # # # # #                     # '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_0.h5',]
# # # # # # # image_tools.split_h5_loop_segments(h5_to_split_list, [1, 1, 1, 99], [bd+'train1',bd+'train2',bd+'train3', bd+'val4'], chunk_size=1000, add_numbers_to_name=False,
# # # # # # #              disable_TQDM=False, set_seed = 0, color_channel=False)
# # # # # # #
# # # # # # #
# # # # # # # import h5py
# # # # # # # with h5py.File(bd+'train1.h5', 'r') as h:
# # # # # # #     print(h['images'][:].shape)
# # # # # # #     print(h['labels'][:].shape)
# # # # # # #     print(sum(h['frame_nums'][:]))
# # # # # # #
# # # # # # #
# # # # # # #
# # # # # # #
# # # # # #
# # # # # #
# # # # # # from whacc import utils
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/ALT_LABELS/train_regular_ALT_LABELS.h5'
# # # # # # keys = utils.print_h5_keys(x, return_list=True)
# # # # # # tmp1 = []
# # # # # # for k in keys:
# # # # # #     tmp1.append(image_tools.get_h5_key_and_concatenate([x], k))
# # # # # #
# # # # # # for k in tmp1:
# # # # # #     print(list(set(k)))
# # # # # #
# # # # # #
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/regular/val_regular.h5'
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/AH0000x000000_5border_single_frame_onset_offset_for_aug_3lag_diff_remove_2_edges_AUG_5.h5'
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/train.h5'
# # # # # # x = l1[0]
# # # # # # x = l2[0]
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/regular/val_regular.h5'
# # # # # # a = image_tools.get_h5_key_and_concatenate([x])
# # # # # # print(list(set(a)))
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # #
# # # # # # def label_naming_shorthand_dict(name_key=None):
# # # # # #     label_naming_shorthand_dict = {
# # # # # #         '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
# # # # # #         '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
# # # # # #         '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
# # # # # #         '[0, 1]- (no touch, touch)': 'regular',
# # # # # #         '[0, 1]- (not offset, offset)': 'only_offset',
# # # # # #         '[0, 1]- (not onset, onset)': 'only_onset',
# # # # # #         '[0, 1, 2, 3]- (no touch, touch, one after onset, offset)': 'overlap_whisker_on-off'}
# # # # # #     if name_key is None:
# # # # # #         return label_naming_shorthand_dict
# # # # # #     else:
# # # # # #         return label_naming_shorthand_dict[name_key]
# # # # # #
# # # # # # def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
# # # # # #     label_key_shorthand = label_naming_shorthand_dict(label_key_name)
# # # # # #     for img_src, lab_src in zip(image_h5_list, label_h5_list):
# # # # # #         utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
# # # # # #
# # # # # # from whacc import image_tools
# # # # # # from whacc import utils
# # # # # # from whacc import analysis
# # # # # # import matplotlib.pyplot as plt
# # # # # #
# # # # # # x = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/3lag/small_train_3lag.h5'
# # # # # #
# # # # # # alt_labels = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/ALT_LABELS/small_train_ALT_LABELS.h5'
# # # # # #
# # # # # # copy_over_new_labels('[0, 1, 2, 3]- (no touch, touch, one after onset, offset)', [x], [alt_labels])
# # # # # #
# # # # # # a = analysis.pole_plot(x)
# # # # # # a.len_plot = 11
# # # # # # a.current_frame = 36
# # # # # # a.plot_it()
# # # # # # plt.ylim([-.5, 6])
# # # # #
# # # # # def search_sequence_numpy(arr, seq, return_type='indices'):
# # # # #     Na, Nseq = arr.size, seq.size
# # # # #     r_seq = np.arange(Nseq)
# # # # #     M = (arr[np.arange(Na - Nseq + 1)[:, None] + r_seq] == seq).all(1)
# # # # #
# # # # #     if return_type == 'indices':
# # # # #         return np.where(M)[0]
# # # # #     elif return_type == 'bool':
# # # # #         return M
# # # # #
# # # # #
# # # # # import numpy as np
# # # # #
# # # # # a = np.asarray([0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1])
# # # # # a = np.asarray([0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1])
# # # # #
# # # # # a1 = search_sequence_numpy(a, np.asarray([0, 0, 1, 0, 0]))
# # # # # a2 = search_sequence_numpy(a, np.asarray([1, 1, 0, 1, 1]))
# # # # #
# # # # # print(a1, a2)
# # # # #
# # # # # np.convolve(a, np.ones(4) / 4)
# # # # #
# # # # # x = np.asarray([0, 1, 0])
# # # # # seg_len = len(x)
# # # # # border = x[0]
# # # # # for k in a1:
# # # # #     k = int(k + np.floor(seg_len / 2))
# # # # #     side_sums = 0
# # # # #     for kk in range(2, 10):
# # # # #         side_sums = a[k - kk] + a[k + kk] + side_sums
# # # # #         print(side_sums / ((kk - 1) * 2))
# # # # #         asdf
# # # # #
# # # # #     sdfadf
# # #
# # # # from whacc import analysis
# # # # pp = analysis.pole_plot('/Users/phil/Dropbox/Colab data/H5_data/OG/AH0407_160613_JC1003_AAAC-006.h5')
# # # # pp.plot_it()
# # # # pp.current_frame=8271
# # # #
# # # #
# # # #
# # # # from whacc import utils
# # # # x  = utils.print_h5_keys('/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/AH0407_160613_JC1003_AAAC_ALT_LABELS.h5', True, False)
# # # # tmp1 = utils.lister_it(x, keep_strings = ['MODEL_2_'])
# # # # print(len(tmp1))
# # #
# # #
# # # from google.colab import drive
# # # import numpy as np
# # # import h5py
# # # import os
# # # from whacc import utils
# # #
# # # drive.mount('/content/gdrive')
# # #
# # # base_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/'
# # # h5_list_to_write = utils.get_h5s(base_dir)
# # # all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/',
# # #                         print_h5_list=False)
# # # h_cont, h_names = utils._get_human_contacts_(all_h5s)
# # # print('--------\npercent fully agree\n--------')
# # # for k in h_cont:
# # #     a = np.mean(np.mean(k, axis=0) == 1)
# # #     b = np.mean(np.mean(k, axis=0) == 0)
# # #     print(np.round(a + b, 4))
# # #
# # #
# # # def add_to_h5(h5_file, key, values, overwrite_if_exists=False):
# # #     all_keys = utils.print_h5_keys(h5_file, return_list=True, do_print=False)
# # #     with h5py.File(h5_file, 'r+') as h:
# # #         if key in all_keys and overwrite_if_exists:
# # #             print('key already exists, overwriting value...')
# # #             del h[key]
# # #             h.create_dataset(key, data=values)
# # #         elif key in all_keys and not overwrite_if_exists:
# # #             print("""key already exists, NOT overwriting value..., \nset 'overwrite_if_exists' to True to overwrite""")
# # #         else:
# # #             h.create_dataset(key, data=values)
# # #
# # #
# # # for kk, h5 in zip(h_cont, all_h5s):
# # #     F_NAME = os.path.basename(h5).split('Phil_')[-1][:-3]
# # #     h52write = utils.lister_it(h5_list_to_write, keep_strings=F_NAME)[0]
# # #     print(h52write)
# # #
# # #     avg_cont = (np.mean(kk, axis=0) > .5) * 1
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
# # #     for k in tmp1 + 1:
# # #         avg_cont[k] = 0
# # #
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 1]))
# # #     for k in tmp1 + 1:
# # #         avg_cont[k] = 1
# # #
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 1, 0]))
# # #     for k in tmp1 + 1:
# # #         avg_cont[k] = 0
# # #     for k in tmp1 + 2:
# # #         avg_cont[k] = 0
# # #
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 0, 1]))
# # #     for k in tmp1 + 1:
# # #         avg_cont[k] = 1
# # #     for k in tmp1 + 2:
# # #         avg_cont[k] = 1
# # #
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
# # #     assert tmp1.size == 0
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 1]))
# # #     assert tmp1.size == 0
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 1, 0]))
# # #     assert tmp1.size == 0
# # #     tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 0, 1]))
# # #     assert tmp1.size == 0
# # #
# # #     utils.add_to_h5(h52write, 'labels', avg_cont, overwrite_if_exists=True)
# # #
# # # utils.make_alt_labels_h5s(base_dir)
# # # alt_label_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/ALT_LABELS/'
# # #
# # # to_pred_h5s = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED'
# # # to_pred_h5s = utils.get_h5s(to_pred_h5s)
# # #
# # # tmp1 = utils.get_h5s(alt_label_dir)
# # # utils.print_h5_keys(tmp1[0])
# # # for h5, h5_dest in zip(tmp1, to_pred_h5s):
# # #     assert os.path.basename(h5) == os.path.basename(h5_dest)
# # #     keys = utils.print_h5_keys(h5, return_list=True, do_print=False)
# # #     for key in keys:
# # #         utils.copy_h5_key_to_another_h5(h5, h5_dest, key)
# # #
# # #
# # #
# # # import numpy as np
# # # import h5py
# # # import os
# # # from whacc import utils, image_tools
# # # import matplotlib.pyplot as plt
# # #
# # # tmp1 = "/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/AH0705_171105_PM0175_AAAB_ALT_LABELS.h5"
# # # a = utils.print_h5_keys(tmp1, 1, 1)
# # # b = image_tools.get_h5_key_and_concatenate([tmp1], a[34])
# # # plt.plot(b[:16000])
# # #
# # #
# # #
# # #
# # #
# # xxx = ['/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/train.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/train_regular.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/val.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/val_regular.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/ALT_LABELS/train_ALT_LABELS.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/train_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/val_regular_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/train_regular_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/ALT_LABELS/val_ALT_LABELS.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff_AUG_AUG.h5.h5']
# #
# #
# # import numpy as np
# #
# # # utils.lister_it(xxx, remove_string=['ALT_LABELS', 'single_frame'])
# # def lister_it(in_list, keep_strings='', remove_string=None):
# #     def index_list_of_strings(in_list2, cmp_string):
# #         return np.asarray([cmp_string in string for string in in_list2])
# #
# #     if isinstance(keep_strings, str): keep_strings = [keep_strings]
# #     if isinstance(remove_string, str): remove_string = [remove_string]
# #
# #     keep_i = np.asarray([False]*len(in_list))
# #     for k in keep_strings:
# #         keep_i = np.vstack((keep_i, index_list_of_strings(in_list, k)))
# #     keep_i = np.sum(keep_i, axis = 0)>0
# #
# #     remove_i = np.asarray([True]*len(in_list))
# #     if remove_string is not None:
# #         for k in remove_string:
# #             remove_i = np.vstack((remove_i, np.invert(index_list_of_strings(in_list, k))))
# #         remove_i = np.product(remove_i, axis = 0)>0
# #
# #     inds = keep_i * remove_i#np.invert(remove_i)
# #     out = np.asarray(in_list)[inds]
# #     return out
# #
# # xxx = ['123', '234', '345', '456', '567', '678']
# # tmp1 = lister_it(xxx, keep_strings=['1', '2', '6'], remove_string=['4'])
# # # tmp1 = lister_it(xxx, keep_strings=['1', '2', '6'], remove_string=['4', '5'])
# # # tmp1 = lister_it(xxx, keep_strings=['1', '2', '6'], remove_string=None)
# # tmp1 = lister_it(xxx, keep_strings='', remove_string='4')
# # tmp1
# #
# #
# #
# # substring_in_list
# # def edit_list_of_strings(list_of_strings, edit_type, edit_string):
# #     if edit_type not in ['keep', 'remove']:
# #         assert False, """edit_type needs to be with 'keep' OR 'remove'"""
# #     x = []
# #     for k in list_of_strings:
# #         if edit_string in k:
# #             x.append(1)
# #         else
# #             x.append(0)
# #
# #
# #
# # def lister_it(in_list, keep_strings=None, remove_string=None):
# #
# #     if isinstance(keep_strings, str):
# #         keep_strings = [keep_strings]
# #     if isinstance(remove_string, str):
# #         remove_string = [remove_string]
# #
# #     if keep_strings is None:
# #         new_list = copy.deepcopy(in_list)
# #     else:
# #         new_list = []
# #         for L in in_list:
# #             for k in keep_strings:
# #                 if k in L:
# #                     new_list.append(L)
# #
# #     if remove_string is None:
# #         new_list_2 = copy.deepcopy(in_list)
# #     else:
# #         new_list_2 = []
# #         for L in new_list:
# #             for k in remove_string:
# #                 if k not in L:
# #                     new_list_2.append(L)
# #     final_list = intersect_lists([new_list_2, new_list])
# #     return final_list
# #
# # from whacc import utils
# # xxx = ['/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/train.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/train_regular.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/val.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/train_3lag_diff.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/val_regular.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/ALT_LABELS/train_ALT_LABELS.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/val_3lag.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/single_frame/train_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/val_regular_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/regular/train_regular_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/ALT_LABELS/val_ALT_LABELS.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag/train_3lag_AUG_AUG.h5.h5',
# #  '/content/data_AH0407_160613_JC1003_AAAC/3lag_diff/val_3lag_diff_AUG_AUG.h5.h5']
# #
# # utils.lister_it(xxx, remove_string=['ALT_LABELS', 'single_frame'])
# from whacc import utils, image_tools, transfer_learning, analysis
# import matplotlib.pyplot as plt
# from tensorflow.keras.layers import Dense, Dropout
# from tensorflow.keras.models import Model
# from tensorflow import keras
# from sklearn.utils import class_weight
# import time
# from pathlib import Path
# import os
# import copy
# import numpy as np
# from tensorflow.keras import applications
# from pathlib import Path
# from google.colab import drive
# import shutil
# import zipfile
# from datetime import datetime
# import pytz
# import json
# drive.mount('/content/gdrive')
# from whacc import model_maker
#
#
# from whacc.model_maker import *
# import itertools
#
# import tensorflow as tf
# from tensorflow import keras
# import matplotlib.pyplot as plt
# import numpy as np
# import h5py
# from whacc import image_tools
# from whacc import utils
# import copy
# import time
# import os
# import pdb
# import glob
# from tqdm.contrib import tzip
# import scipy.io as spio
# import glob
# import h5py
# from tqdm.notebook import tqdm
# from matplotlib import cm
#
#
# target_path = "/content/DATA_FULL/"
# source_path = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/DATA_FULL/"
# Path(target_path).mkdir(parents = True, exist_ok = True)
# sync(source_path, target_path, 'sync', only=pattern)
# h5_IMG_SRC_ALL = utils.get_h5s(target_path)
#
# base_dir = "/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/"
# h5_to_write_all = utils.get_h5s(base_dir)
# alt_label_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/ALT_LABELS/'
#
#
# key_name = 'acc_test'
# max_or_min = 'max'
#
# #####
# key_names = [] # it take a long time to transfer the labels so this groups them together so
# # we dont have to do them all on each iteration
#
#
# # for iiii, model_ind in enumerate(tqdm(model_inds_sorted_by_label_type)):
# for iiii, data in enumerate(tqdm(all_data)):
#
#   if True:
#     # data = all_data[model_ind]
#     best_epochs = sorted_loadable_epochs(data, key_name, max_or_min)
#     reload_model_epoch = best_epochs[0]
#     D = data['info']
#
#     model = re_build_model(D['model_name_str'], D['class_numbers'],
#                         base_learning_rate=D['base_learning_rate'],
#                         dropout_val=D['dropout_val'], IMG_SIZE=D['IMG_SIZE'])
#     model.load_weights(D['epoch_dict'][reload_model_epoch]) # load model weights
#     label_ind = np.where(np.asarray(D['label_key_name'])  == np.asarray(list(model_maker.label_naming_shorthand_dict().keys())))[0][0]
#     pred_key_save_name = 'MODEL_2_' + data['full_name'] + '__' + key_name + ' ' + max_or_min + '__epoch '+ str(reload_model_epoch)+ '__L_ind'+ str(label_ind)+'__LABELS'
#
#     for h5_to_write in tqdm(h5_to_write_all):
#
#       keep_strings = [D['image_source_h5_directory_ending'], os.path.basename(h5_to_write)[:-3]]
#       h5_IMG_SRC = utils.lister_it(h5_IMG_SRC_ALL, keep_strings=keep_strings, remove_string=[''])
#       img_gen = image_tools.ImageBatchGenerator(50, [h5_IMG_SRC],label_key = D['label_key_name'] )
#       pred = model.predict(img_gen) #predict
#       with h5py.File(h5_to_write, 'r+') as hf:
#         try:
#           hf.create_dataset(pred_key_save_name, data=pred)
#         except:
#           del hf[pred_key_save_name]
#           time.sleep(10)
#           hf.create_dataset(pred_key_save_name, data=pred)
#
#
# #
# #
# # base_dir = "/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/"
# # h5_img_SRS_ALL = utils.get_h5s(base_dir)
# # alt_label_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/ALT_LABELS/'
# #
# #
# # key_name = 'acc_test'
# # max_or_min = 'max'
# #
# # #####
# # key_names = [] # it take a long time to transfer the labels so this groups them together so
# # # we dont have to do them all on each iteration
# #
# # for data in (all_data):
# #   key_names.append(data['info']['label_key_name'])
# # model_inds_sorted_by_label_type = np.argsort(key_names)
# #
# # kn = ''
# # for iiii, model_ind in enumerate(tqdm(model_inds_sorted_by_label_type)):
# #   if 100 == 100:
# #     data = all_data[model_ind]
# #     best_epochs = sorted_loadable_epochs(data, key_name, max_or_min)
# #     reload_model_epoch = best_epochs[0]
# #     D = data['info']
# #
# #     model = re_build_model(D['model_name_str'], D['class_numbers'],
# #                         base_learning_rate=D['base_learning_rate'],
# #                         dropout_val=D['dropout_val'], IMG_SIZE=D['IMG_SIZE'])
# #     model.load_weights(D['epoch_dict'][reload_model_epoch]) # load model weights
# #     label_ind = np.where(np.asarray(D['label_key_name'])  == np.asarray(list(model_maker.label_naming_shorthand_dict().keys())))[0][0]
# #     pred_key_save_name = 'MODEL_2_' + data['full_name'] + '__' + key_name + ' ' + max_or_min + '__epoch '+ str(reload_model_epoch)+ '__L_ind'+ str(label_ind)+'__LABELS'
# #
# #     for h5_image_SRC in tqdm(h5_img_SRS_ALL):
# #       if kn != D['label_key_name']: # if new label type copy over labels
# #         label_src = utils.get_files(alt_label_dir, os.path.basename(h5_image_SRC)[:-13]+'*')[0]
# #         local_dict = model_maker.copy_over_new_labels(D['label_key_name'], [h5_image_SRC], [label_src])
# #       img_gen = image_tools.ImageBatchGenerator(50, [h5_image_SRC])
# #       pred = model.predict(img_gen) #predict
# #       with h5py.File(h5_img_file, 'r+') as hf:
# #         try:
# #           hf.create_dataset(pred_key_save_name, data=pred)
# #         except:
# #           del hf[pred_key_save_name]
# #           time.sleep(10)
# #           hf.create_dataset(pred_key_save_name, data=pred)
# #     kn = D['label_key_name']
#
# # download transformed full sessions
#
# h5_to_write_all = ['/Users/phil/Downloads/trash_test copy/3lag/small_test_3lag.h5',
# '/Users/phil/Downloads/trash_test copy/3lag_diff/small_test_3lag_diff.h5']
# pred_key_save_name = 'labelss'
# key_not_in_h5 = []
# for h5_to_write in h5_to_write_all:
#   tmp1 = utils.print_h5_keys(h5_to_write, return_list=True, do_print=False)
#   key_not_in_h5.append(pred_key_save_name not in tmp1)
# any(key_not_in_h5)
# # all(already_done)
#
#
#
# import numpy as np
# tmp1 = np.load('/Users/phil/Downloads/all_data.npy')
#
# M2 = np.load('/Users/phil/Downloads/all_data.npy', allow_pickle=True)[()]
#
#
#
#
# import os
# import shutil
# import os
# xxx = "/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/"
# tmp1 = next(os.walk(xxx))[1]
#
#
# xxx = "/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/"
# tmp1 = next(os.walk(xxx))[1]
# for k in tmp1:
#     asdf
#     shutil.make_archive('/Users/phil/Downloads/'+k,
#                         'zip',
#                         xxx,
#                         k)
#
# xxx = "/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/"
# tmp1 = utils.get_h5s(xxx)
# tmp1
# for k in tmp1:
#     shutil.make_archive('/Users/phil/Downloads/'+os.path.basename(k),
#                         'zip',
#                         xxx,
#                         os.path.basename(k))
#
#
#
#
# for k in utils.get_h5s('/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/', 0):
#     try:
#         # print(k)
#         tmp1 = utils.print_h5_keys(k, 1, 0)
#         # print('\n\n\n\n')
#     except:
#         print(k)
#
#
#
#
# xxx = "/Users/phil/Downloads/untitled folder 3/"
# tmp1 = utils.get_h5s(xxx)
# tmp1
# for k in tmp1:
#     shutil.make_archive('/Users/phil/Downloads/'+os.path.basename(k),
#                         'zip',
#                         xxx,
#                         os.path.basename(k))
#
# utils.print_h5_keys(tmp1[0])
# for k in utils.get_h5s('/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/', 0):
#     utils.print_h5_keys(k)
#     sadfsdf
#
#
# utils.print_h5_keys(utils.get_h5s('/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/', 0)[2])
#
#
#
#
#
#
#
#
#
# def func(arg1, arg2, arg3):
#     print('arg1 =', arg1)
#     print('arg2 =', arg2)
#     print('arg3 =', arg3)
#
# d = {'arg1': 'one', 'arg2': 'two', 'arg3': 'three'}
#
# func(**d)
# # arg1 = one
# # arg2 = two
# # arg3 = three
#
# func(**{'arg1': 'one', 'arg2': 'two', 'arg3': 'three'})
# # arg1 = one
# # arg2 = two
# # arg3 = three
#
#
#
#
# import shutil
# shutil.move(source,destination)
#
# # find /"/Users/phil/Downloads/" -mtime +180 -size +1G
#

from whacc import utils, image_tools
import matplotlib.pyplot as plt
import numpy as np
x = '/Users/phil/Downloads/untitled folder 2/AH0000x000000_small_tester.h5'
x = '/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Jon/AH0667/170317/AH0667x170317.h5'
utils.print_h5_keys(x)
max_val_stack = image_tools.get_h5_key_and_concatenate(x, 'max_val_stack')
locations_x_y = image_tools.get_h5_key_and_concatenate(x, 'locations_x_y')
trial_nums_and_frame_nums = image_tools.get_h5_key_and_concatenate(x, 'trial_nums_and_frame_nums')
frame_nums = trial_nums_and_frame_nums[1, :].astype(int)

plt.plot(max_val_stack)

loc_x = locations_x_y[:, 0]
loc_y = locations_x_y[:, 1]

max_ind = np.argmax(max_val_stack)

mid_x = []
lo_segs = []
all_max_inds = []
all_maxes = []
cnt=0
for k1, k2 in utils.loop_segments(frame_nums):
    # print(k1, k2)
    tmp1 = max_val_stack[k1:k2]
    tmp1 = tmp1 - tmp1[0]
    cnt+=1000000
    plt.plot(tmp1+cnt)
    all_max_inds.append(np.argmax(max_val_stack[k1:k2]))
    all_maxes.append(np.max(max_val_stack[k1:k2]))
    # print(max_val_stack[k1:k2][all_max_inds[-1]])
    lo_segs.append([k1, k2])
    mid_x.append(loc_x[k1:k2][2000])
lo_segs = np.asarray(lo_segs)

tmp_segs = utils.loop_segments(frame_nums)
plt.figure()
for k in np.argsort(mid_x):
    k1 = lo_segs[k, 0]
    k2 = lo_segs[k, 1]
    tmp1 = max_val_stack[k1:k2]
    tmp1 = tmp1 - tmp1[0]
    cnt+=1000000
    plt.plot(tmp1+cnt)
    # if k1 == 0:
    #     plt.plot(tmp1+cnt, '--')
    # else:
    #     plt.plot(tmp1+cnt)
    all_max_inds.append(np.argmax(max_val_stack[k1:k2]))
    all_maxes.append(np.max(max_val_stack[k1:k2]))
    # print(max_val_stack[k1:k2][all_max_inds[-1]])








trial_ind = np.where(max_ind<np.cumsum(frame_nums))[0][0]

import os
import glob

import cv2
import numpy as np
import time
import re
import h5py
import matplotlib.pyplot as plt
from functools import partial
from tqdm import tqdm
from PIL import Image
from whacc.image_tools import h5_iterative_creator
from sklearn.preprocessing import normalize


def track_h5(template_image, h5_file, match_method='cv2.TM_CCOEFF', ind_list=None):
    with h5py.File(h5_file, 'r') as h5:
        if isinstance(template_image, int):  # if termplate is an ind to the images in the h5
            template_image = h5['images'][template_image, ...]
        if ind_list is None:
            ind_list = range(len(h5['labels'][:]))
        # width and height of img_stacks will be that of template (61x61)
        print(template_image.shape)
        w, h = template_image.shape[0:2]
        max_match_val = []

        method = eval(match_method)
        for frame_i in tqdm(ind_list):
            img = h5['images'][frame_i, ...]
            # Apply template Matching
            res = cv2.matchTemplate(img, template_image, method)
            min_val, max_val, min_loc, top_left = cv2.minMaxLoc(res)
            max_match_val.append(max_val)
            top_left = np.flip(np.asarray(top_left))
    return max_match_val


h5_file = '/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Jon/AH0667/170317/AH0667x170317.h5'
meth_dict = dict()
meth_dict['h5_file'] = h5_file
ind_list = range(8000)
for template_image_ind in [0, 2000]:
    for method in ['TM_SQDIFF', 'TM_SQDIFF_NORMED', 'TM_CCORR', 'TM_CCORR_NORMED', 'TM_CCOEFF', 'TM_CCOEFF_NORMED']:
        max_match_val_new = track_h5(template_image_ind, h5_file, match_method='cv2.'+method, ind_list=ind_list)
        meth_dict['ind_'+str(template_image_ind)+'_'+ method] = max_match_val_new


fig, ax = plt.subplots(nrows=2, ncols=3,sharex=True, sharey=False)
ax_list = fig.axes
cnt = -1
for k in meth_dict:
    if 'h5_file' not in k:
        cnt+=1
        if len(ax_list) == cnt:
            cnt = 0
            fig, ax = plt.subplots(nrows=2, ncols=3,sharex=True, sharey=False)
            ax_list = fig.axes
        ax1 = ax_list[cnt]
        ax1.set_title(k)
        # plt.title(k)
        for k1, k2 in utils.loop_segments(frame_nums):
            try:
                x = np.asarray(meth_dict[k][k1:k2])
                ax1.plot(x-x[0],linewidth=.3, alpha = .3)
            except:
                break



plt.figure()
ax_list = fig.axes
cnt = -1
for k in meth_dict:
    if 'h5_file' not in k and 'NORM' in k:
        cnt+=1
        if len(ax_list) == cnt:
            cnt = 0
            plt.figure()
        for k1, k2 in utils.loop_segments(frame_nums):
            try:
                x = np.asarray(meth_dict[k][k1:k2])
                plt.plot(x-x[0],linewidth=.3, alpha = .3)
            except:
                break

# ind 1 is fractionally bigger on whisker in frames for the first trial where the actaul frame comes from
# but way less for the other trials, considering I am using images from each unique trial ths may be an
# advantage
names = ['TM_SQDIFF', 'TM_SQDIFF_NORMED', 'TM_CCORR', 'TM_CCORR_NORMED', 'TM_CCOEFF', 'TM_CCOEFF_NORMED']
names_ind = 1
frame_num = 2000
x = np.asarray(meth_dict['ind_'+str(frame_num)+'_'+names[names_ind]])
x = x-np.min(x)
x = x/np.max(x)
# x = normalize([x], )[0]
plt.plot(x, linewidth = .3)


names_ind = 3
frame_num = 2000
x = np.asarray(meth_dict['ind_'+str(frame_num)+'_'+names[names_ind]])
x = x*-1
x = x-np.min(x)
x = x/np.max(x)
# x = normalize([x], )[0]
plt.plot(x, linewidth = .3)


names_ind = 5
frame_num = 2000
x = np.asarray(meth_dict['ind_'+str(frame_num)+'_'+names[names_ind]])
x = x*-1
x = x-np.min(x)
x = x/np.max(x)
# x = normalize([x], )[0]
plt.plot(x, linewidth = .3)

#
#
# max_match_val_new = np.asarray(max_match_val_new)
# plt.figure()
# plt.title('testing indo 0 ')
# for k in np.argsort(mid_x):
#     k1 = lo_segs[k, 0]
#     k2 = lo_segs[k, 1]
#     tmp1 = max_match_val_new[k1:k2]
#     tmp1 = tmp1 - tmp1[0]
#     cnt+=1000000
#     plt.plot(tmp1+cnt)
#     # if k1 == 0:
#     #     plt.plot(tmp1+cnt, '--')
#     # else:
#     #     plt.plot(tmp1+cnt)
#     all_max_inds.append(np.argmax(max_val_stack[k1:k2]))
#     all_maxes.append(np.max(max_val_stack[k1:k2]))
#     # print(max_val_stack[k1:k2][all_max_inds[-1]])
#
#



from setuptools import setup
from setuptools import find_packages
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='whacc',
      version='0.0.101',
      author="Phillip Maire",
      license='MIT',
      description='largely automatic and customizable pipeline for creating a CNN to predict whiskers contacting objects',
      packages=find_packages(),
      author_email='phillip.maire@gmail.com',
      long_description=long_description,
      long_description_content_type="text/markdown",
      zip_safe=False,
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"],
      python_requires='>=3.6',
      install_requires=[
       # "pyicu",
       "natsort==7.1.1"])

# package_data={
    #     'whacc': ['datasets/*.h5',
    #               'model_checkpoints/*',
    #               'model_checkpoints/model_201008/*',
    #               'model_checkpoints/model_201008/variables/*',
    #               'model_checkpoints/model_201008/assets/*'],
    # },
# include_package_data=True,






from whacc import utils, transfer_learning

import matplotlib.pyplot as plt
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras

from sklearn.utils import class_weight

import os
import copy
import numpy as np
from tensorflow.keras import applications
from pathlib import Path

import shutil
import zipfile
from datetime import datetime
import pytz
import json


# import sys
# sys.path.append('/opt/conda/lib/python3.7/site-packages')
# bd2 = '/home/jupyter/'


def build_model(info_dict, labels, model_name_str, base_learning_rate=0.00001, dropout_val=None, class_numbers=None,
                IMG_SIZE=96):
    if class_numbers is None:
        class_numbers = np.unique(labels)
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
    class_weights = {i: rebalance[i] for i in class_numbers}
    wrap_vars_list = ['class_numbers',
                      'num_classes',
                      'base_learning_rate',
                      'model_name_str',
                      'IMG_SIZE',
                      'dropout_val']
    for k in wrap_vars_list:
        info_dict[k] = eval(k)
    info_dict['class_weights'] = str(class_weights)
    return model, class_weights, info_dict


def label_naming_shorthand_dict(name_key=None):
    label_naming_shorthand_dict = {
        '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
        '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
        '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
        '[0, 1]- (no touch, touch)': 'regular',
        '[0, 1]- (not offset, offset)': 'only_offset',
        '[0, 1]- (not onset, onset)': 'only_onset',
        '[0, 1, 2, 3]- (no touch, touch, one after onset, offset)': 'overlap_whisker_on-off'}
    if name_key is None:
        return label_naming_shorthand_dict
    else:
        return label_naming_shorthand_dict[name_key]


def info_dict_wrapper(info_dict, local_dict):
    for k in local_dict.keys():
        info_dict[k] = local_dict[k]
    return info_dict


def make_initial_folder(all_models_directory, unique_h5_train_val_dir):
    single_frame_dir = all_models_directory + os.sep + unique_h5_train_val_dir + os.sep + 'data' + os.sep + 'single_frame'
    Path(single_frame_dir).mkdir(parents=True, exist_ok=True)
    return locals()
    # return single_frame_dir


def get_automated_model_info(BASE_H5, image_source_h5_directory_ending, test_data_dir):
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + "data" + a
    base_dir_all_h5s = BASE_H5 + a + 'data' + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    labels_dir = base_data_dir + a + "ALT_LABELS" + a
    print('\nFOR LABELS,0 is train set, 1 is val set')
    label_h5_list = utils.get_h5s(labels_dir)
    print('\nSelect from the following label structures...')
    label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()


def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
    return locals()


# get list of pre trained models to choose from
def get_keras_model_names():
    names_, types_ = utils.get_class_info(applications, return_name_and_type=True)
    model_names = np.asarray(names_)['function' == np.asarray(types_)]
    utils.print_list_with_inds(model_names)
    return model_names


def make_model_save_directory(info_dict, make_folder=True):
    naming_list = ['model_name_str', 'image_source_h5_directory_ending', 'label_key_shorthand', 'todays_version']
    model_save_dir: str = copy.deepcopy(info_dict['BASE_H5'])
    for k in naming_list:
        model_save_dir += os.sep + info_dict[k] + os.sep
    info_dict['model_save_dir'] = model_save_dir
    info_dict['model_save_dir_checkpoints'] = model_save_dir + os.sep + 'checkpoints'
    if make_folder:
        Path(info_dict['model_save_dir_checkpoints']).mkdir(parents=True, exist_ok=True)
    return model_save_dir


def basic_callbacks(save_checkpoint_filepath, monitor='val_loss', patience=10,
                    save_best_only=False, save_weights_only=True, save_freq="epoch", period=1):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, patience=patience))
    add_path_name = "{loss:.8f}_{epoch:04d}_cp.hdf5"
    add_path_name = "{epoch:04d}_cp.hdf5"
    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath + os.sep + add_path_name,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq,
        period=period))
    return callbacks


def unzip_and_place_h5s(bd, do_delete_zips=False):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    shutil.copytree(bd, bd2)
    a = utils.get_files(bd2, '*.zip')
    for k in a:
        with zipfile.ZipFile(k, 'r') as zip_ref:
            zip_ref.extractall(os.path.dirname(k))
        if do_delete_zips:
            os.remove(k)


def change_to_local_dir(bd):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    return bd2


def change_to_gdrive_dir(bd):
    bd2 = '/content/gdrive/My Drive/'.join(bd.split('/content/'))
    return bd2


def pack_training_info(save_and_plot_history_var):
    training_info = dict()
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    for k in training_info.keys():
        if 'numpy.ndarray' in str(type(training_info[k])):
            training_info[k] = training_info[k].tolist()
    return training_info


def save_info_dict(info_dict_tmp):
    for k in info_dict_tmp.keys():
        if 'numpy.ndarray' in str(type(info_dict_tmp[k])):
            info_dict_tmp[k] = info_dict_tmp[k].tolist()
    with open(info_dict_tmp['model_save_dir'] + 'info_dict' + '.json', 'w') as f:
        json.dump(info_dict_tmp, f)


def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = transfer_learning.make_text_for_fig(training_info)
    training_info = transfer_learning.pack_training_info(training_info, save_and_plot_history_1)
    # save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    transfer_learning.plot_train_hist(training_info, [1], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC' + '.png', bbox_inches="tight")
    transfer_learning.plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS' + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'model_eval_each_epoch' + '.json', 'w') as f:
        json.dump(training_info, f)


def re_build_model(model_name_str, class_numbers, base_learning_rate=0.00001, dropout_val=None, IMG_SIZE=96,
                   labels=None):
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model


def reload_info_dict(base_dir):
    info_dict_path = utils.get_files(base_dir, '*info_dict.json')[0]
    with open(info_dict_path, 'r') as f:
        D = json.load(f)
    # replace the directory to the main directory
    x = 'model_testing'
    old_base_dir = ''
    for k in D.keys():
        try:
            if x in D[k]:
                D[k] = base_dir.split(x)[0] + x + D[k].split(x)[-1]
                old_base_dir = D[k].split(x)[0]
        except:
            pass
    D['old_base_dir'] = old_base_dir
    D['checkpoints'] = utils.get_files(D['model_save_dir_checkpoints'], '*hdf5')
    return D


def load_model_data(all_models_directory):
    data_files = utils.get_files(all_models_directory, '*model_eval_each_epoch.json')
    to_split = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'
    all_data = []
    for i, k in enumerate(data_files):
        if '/small_h5s/' not in k:
            with open(k, 'r') as f:
                a = json.load(f)
            a['all_logs'] = np.asarray(a['all_logs'])
            new_log_names = []
            for k2 in a['logs_names']:
                new_log_names.append(k2.split('bool_')[-1])
            a['logs_names'] = new_log_names
            a['logs_names'] = np.asarray(a['logs_names'])

            k.split(to_split)[-1]
            a['full_name'] = '__'.join(' '.join(k.split(to_split)[-1].split('/2021')[0].split('_')).split('/'))
            a['dir'] = os.path.dirname(k)
            all_data.append(a)
    return all_data
# def reload_info(base_dir):
#   s = os.sep
#   b = base_dir.split('all_models'+s)[-1].split(s)
#   reload_dict = {'data_type':b[0],
#                 'model_name':b[1],
#                 'img_type':b[2],
#                 'label_name_shorthand':b[3],
#                 'date_string':b[4]}
#
#   label_name_dict = label_naming_shorthand_dict()
#   for k in label_name_dict.keys():
#     if reload_dict['label_name_shorthand'] == label_name_dict[k]:
#       reload_dict['label_type'] = k
#       class_numbers = eval(k.split('- ')[0])
#   reload_dict['class_numbers'] = class_numbers
#   return reload_dict



from keras.preprocessing.image import ImageDataGenerator
from imgaug import augmenters as iaa  # optional program to further augment data
from tqdm import tqdm
from whacc.model_maker import *
from whacc import image_tools
from whacc import utils
import os
from tqdm.contrib import tzip
import h5py


def make_all_h5s_from_subset(base_dir_all, subset_h5_file, num_to_augment=1):
    def get_h5_key_and_concatenate(h5_list, key_name='labels'):
        """
        simply extract and concatenate all of one key "key_name" from many H5 files, I use it to get balance the data touch
        and not touch frames when training a model with a list of different H5 files
        Parameters
        ----------
        h5_list : list
            list of full paths to H5 file(s).
        key_name : str
            default 'labels', the key to get the data from the H5 file

        """
        h5_list = utils.make_list(h5_list, suppress_warning=True)
        for i, k in enumerate(h5_list):
            with h5py.File(k, 'r') as h:
                if i == 0:
                    out = np.asarray(h[key_name][:])
                else:
                    out = np.concatenate((out, h[key_name][:]))
        return out

    def repeat_labels(h5_file, label_key):  # with a shape test
        with h5py.File(h5_file, 'r+') as h:
            num_repeats = h['images'][:].shape[0] / h[label_key][:].shape[0]
            if num_repeats > 1 and round(num_repeats) == num_repeats:
                labels = h[label_key][:]
                rep_labels = np.repeat(labels, num_repeats, axis=0)
                del h[label_key]
                h[label_key] = rep_labels

    def foo_get_frame_nums_from_all_inds(h5_subset_file):
        tmp1 = image_tools.get_h5_key_and_concatenate([h5_subset_file], 'all_inds')
        tmp1, tmp2 = utils.group_consecutives(tmp1)
        frame_nums = []
        for k in tmp2:
            frame_nums.append(len(k))
        return frame_nums

    # **********$%$%$%$%$%%$
    # these indacate areas where user may want to customize like make a test set for example
    # **********$%$%$%$%$%%$

    tmp_h5s = base_dir_all + '/tmp_h5s/'
    subset_h5s = [subset_h5_file]
    for h5_subset_file in tqdm(subset_h5s):
        a = h5_subset_file.split('/')[-1].split('RETRAIN_')[-1].split('.h5')[0]
        base_h5 = base_dir_all + '/DATA/data_' + a
        single_frame_h5s = base_h5 + '/single_frame/'
        Path(tmp_h5s).mkdir(parents=True, exist_ok=True)
        Path(single_frame_h5s).mkdir(parents=True, exist_ok=True)
        # vvvvv add the frame nums to the H5 file
        frame_nums = foo_get_frame_nums_from_all_inds(h5_subset_file)

        utils.add_to_h5(h5_subset_file, 'frame_nums', frame_nums, overwrite_if_exists=True)
        # ^^^^^ add the frame nums to the H5 file
        # vvvvvv single frame and then split
        utils.reduce_to_single_frame_from_color(h5_subset_file, tmp_h5s + 'temp.h5')

        # split_h5s = image_tools.split_h5([tmp_h5s + 'temp.h5'],
        #                                    split_percentages=[.8, .2],  # **********$%$%$%$%$%%$
        #                                    temp_base_name=[single_frame_h5s + 'train',
        #                                                    single_frame_h5s + 'val'],
        #                                    chunk_size=10000,
        #                                    add_numbers_to_name=False,
        #                                    disable_TQDM=True,
        #                                    set_seed=0,
        #                                    color_channel=False)

        split_h5s = image_tools.split_h5_loop_segments([tmp_h5s + 'temp.h5'],
                                                       split_percentages=[.8, .2],  # **********$%$%$%$%$%%$
                                                       temp_base_name=[single_frame_h5s + 'train',
                                                                       single_frame_h5s + 'val'],
                                                       # **********$%$%$%$%$%%$
                                                       chunk_size=10000,
                                                       add_numbers_to_name=False,
                                                       disable_TQDM=True,
                                                       set_seed=0,
                                                       color_channel=False,
                                                       force_random_each_frame=False)
        # ^^^^^^ single frame and then split
        # vvvvvvvv convert to all types
        utils.make_all_H5_types(
            single_frame_h5s)  # auto generate all the h5 types using a single set of flat (no color) image H5
        utils.make_alt_labels_h5s(single_frame_h5s)  # auto generate the different types of labels
        # ^^^^^^^^ convert to all types
        # vvvvv AUGMENT #**********$%$%$%$%$%%$
        datagen = ImageDataGenerator(rotation_range=360,  #
                                     width_shift_range=.3,  #
                                     height_shift_range=.3,  #
                                     shear_range=.00,  #
                                     zoom_range=.25,
                                     brightness_range=[0.4, 1.4])  #
        gaussian_noise = iaa.AdditiveGaussianNoise(loc=0, scale=3)
        num_aug = 1  # DO NOT CHANGE THIS CHANGE num_to_augment, this preserves the order of the images to allow image subtraction
        xxx = utils.get_h5s(base_h5)
        xxx = utils.lister_it(xxx, remove_string=['ALT_LABELS', 'single_frame'])

        for each_h5 in xxx:
            if 'images' in utils.print_h5_keys(each_h5, True, False):
                combine_list = []
                combine_list.append(each_h5)

                shutil.rmtree(tmp_h5s, ignore_errors=True)
                Path(tmp_h5s).mkdir(parents=True, exist_ok=True)
                for ii in range(num_to_augment):
                    new_H5_file = each_h5.split('.')[0] + '_AUG_' + str(
                        ii) + '.h5'  # create new file name based on teh original H5 name
                    new_H5_file = tmp_h5s + os.path.basename(new_H5_file)
                    combine_list.append(
                        new_H5_file)  # all h5s to combine into one after all augmented files have been created
                    h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
                                                                 close_and_open_on_each_iteration=True,
                                                                 color_channel=True)

                    with h5py.File(each_h5, 'r') as hf:
                        for image, label in tzip(hf['images'][:], hf['labels'][:]):
                            aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
                            aug_img_stack = gaussian_noise.augment_images(aug_img_stack)  # optional
                            h5creator.add_to_h5(aug_img_stack[:, :, :, :], labels_stack)
                    utils.copy_h5_key_to_another_h5(each_h5, new_H5_file, 'frame_nums',
                                                    'frame_nums')  # copy the frame nums to the aug files
                # combine all the
                final_name = each_h5.split('.h5')[0] + '_AUG.h5'
                image_tools.split_h5_loop_segments(combine_list,  # combine all the
                                                   [1],
                                                   final_name,
                                                   add_numbers_to_name=False,
                                                   set_seed=0,
                                                   color_channel=True)
                with h5py.File(final_name, 'r+') as h:
                    h['multiplier'][:] = num_to_augment + 1
    # shutil.rmtree(tmp_h5s)
    shutil.rmtree(tmp_h5s, ignore_errors=True)
    """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
    """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
    """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
    """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
    """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

    # def test_equal(h5_file, label_key):  # with a shape test
    #     with h5py.File(h5_file, 'r') as h:
    #         num_repeats = h['images'][:].shape[0] / h[label_key][:].shape[0]
    #     return num_repeats == 1

    x = base_dir_all + '/DATA/'
    for k in next(os.walk(x))[1]:
        a = utils.get_h5s(x + os.sep + k)
        a = utils.lister_it(a, keep_strings='train')
        src_h5 = utils.lister_it(a, '/ALT_LABELS/', remove_string=['single_frame'])[0]
        dest_h5s = utils.lister_it(a, remove_string=['single_frame', '/ALT_LABELS/'])
        for src_label_key in utils.print_h5_keys(src_h5, 1):
            for dest_H5 in dest_h5s:
                if src_label_key in utils.print_h5_keys(dest_H5, 1, 0):
                    with h5py.File(dest_H5, 'r+') as h:
                        del h[src_label_key]
                utils.copy_h5_key_to_another_h5(src_h5, dest_H5, src_label_key, src_label_key)
                repeat_labels(dest_H5, src_label_key)

    for k in next(os.walk(x))[1]:
        a = utils.get_h5s(x + os.sep + k)
        a = utils.lister_it(a, keep_strings='val')
        src_h5 = utils.lister_it(a, '/ALT_LABELS/', remove_string=['single_frame'])[0]
        dest_h5s = utils.lister_it(a, remove_string=['single_frame', '/ALT_LABELS/'])
        for src_label_key in utils.print_h5_keys(src_h5, 1):
            for dest_H5 in dest_h5s:
                if src_label_key in utils.print_h5_keys(dest_H5, 1, 0):
                    with h5py.File(dest_H5, 'r+') as h:
                        del h[src_label_key]
                utils.copy_h5_key_to_another_h5(src_h5, dest_H5, src_label_key, src_label_key)
                repeat_labels(dest_H5, src_label_key)


base_dir_all = '/Volumes/GoogleDrive/My Drive/Colab data/test_AH1030_subset/'  # where to put the files (will create this folder automatically)
base_dir_all = '/Volumes/GoogleDrive/My Drive/Colab data/test_AH1030_half_data/'  # where to put the files (will create this folder automatically)
subset_h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/data_ANM234232_140120_AH1030_AAAA_a/regular/ANM234232_140120_AH1030_AAAA_a_regular.h5'
num_to_augment = 1
# subset_h5_file = '/Volumes/GoogleDrive/My Drive/Colab data/test_AH1030_subset/ANM234232_140120_AH1030_AAAA_a_subset.h5'
make_all_h5s_from_subset(base_dir_all, subset_h5_file, num_to_augment=num_to_augment)

tmp1 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/data_ANM234232_140120_AH1030_AAAA_a/3lag/ANM234232_140120_AH1030_AAAA_a_3lag.h5'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/test_AH1030_half/'
split_h5s = image_tools.split_h5_loop_segments([tmp1],
                                               split_percentages=[.5, .5],  # **********$%$%$%$%$%%$
                                               temp_base_name=[bd + 'train',
                                                               bd + 'val'],
                                               # **********$%$%$%$%$%%$
                                               chunk_size=10000,
                                               add_numbers_to_name=False,
                                               disable_TQDM=False,
                                               set_seed=0,
                                               color_channel=True,
                                               force_random_each_frame=False)

# h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED/ANM234232_140120_AH1030_AAAA_a_ALT_LABELS.h5'
# real_bool = image_tools.get_h5_key_and_concatenate(h5_file, '[0, 1]- (no touch, touch)')




# -*- coding: utf-8 -*-
"""COMPILE pole_tracker_for_phil_ V_1.2.0.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1jR9Q1z4fIQY37rCOUgWUx8WyBO_lo_Zx
"""

import os
import glob

import cv2
import numpy as np
import time
import re
import h5py
import matplotlib.pyplot as plt
from functools import partial
from tqdm import tqdm
from PIL import Image
from whacc.image_tools import h5_iterative_creator


# import pdb
# from whacc import utils
def isnotebook():
    try:
        c = str(get_ipython().__class__)
        shell = get_ipython().__class__.__name__
        if 'colab' in c:
            return True
        elif shell == 'ZMQInteractiveShell':
            return True  # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False  # Probably standard Python interpreter


if isnotebook():
    from tqdm.notebook import tqdm
else:
    from tqdm import tqdm
# tqdm = partial(tqdm, position=0, leave=True)

################### drive.mount('/content/gdrive')

# sorts the videos the way the OS does, this way the files will be in the ocrrect order
# import pkg_resources
# pkg_resources.require("natsort==7.1.1")
import natsort
# !pip install natsort==7.1.1 # this version has "os_sorted"
# !pip install pyicu # needed to make sure the sorting is correct (strong recommend in the documentation)
from natsort import os_sorted

"""## Build Class for template and frame matching

"""


class PoleTracking():

    def __init__(self, video_directory, template_png_full_name=None, use_narrow_search_to_speed_up=False):
        """

        Parameters
        ----------
        video_directory : str
            full path to the directory with MP4s
        template_png_full_name : numpy array
            template image HxWx3 (can be left blank and use method "cut_out_pole_template" to make a new template and use
            "save_template_img" method to save the template image
        use_narrow_search_to_speed_up : bool
            used to make pole tracker faster. after finding the first frame, it searches in a limited space around that
            frame. based on "inflation" variable in "crop_image_from_top_left" method below
        """
        self.video_directory = video_directory
        self.video_files = os_sorted(glob.glob(os.path.join(video_directory, '*.mp4')))
        self.base_names = [os.path.basename(n).split('.')[0] for n in self.video_files]
        self.use_narrow_search_to_speed_up = use_narrow_search_to_speed_up
        if template_png_full_name is not None:
            self.load_template_img(template_png_full_name)

    @staticmethod
    def crop_image_from_top_left(im, crop_top_left, size_crop, inflation=1):
        """This is an accessory function to track to improve tracking speed. This crops the initial large image into a smaller one, based on the inflation rate.
        Inflation rate of 3 = 3 x 3 template image size around the first guessed pole location.

        Parameters
        ----------
        im :
            
        crop_top_left :
            
        size_crop :
            
        inflation :
             (Default value = 3)

        Returns
        -------

        """
        inflation_shift = np.floor((np.asarray(size_crop) * (inflation - 1)) / 2).astype(int)

        size_crop = np.asarray(size_crop) * inflation
        imshape = np.asarray(im.shape[:2])

        # adjust for inflation and make copies of origianl crop inds
        crop_top_left = crop_top_left - inflation_shift
        crop_top_left2 = crop_top_left.copy()
        crop_bottom_right = crop_top_left + np.asarray(size_crop)
        crop_bottom_right2 = crop_bottom_right.copy()

        # adjust crop in case of out of bounds
        crop_top_left[crop_top_left < 0] = 0
        crop_bottom_right[crop_bottom_right > imshape] = np.asarray(imshape)[crop_bottom_right > imshape]

        # for placing cropped image in random noise image tensor (we do this in case of out of bounds issues all outputs are the same size)
        TL_adj = crop_top_left - crop_top_left2

        c_cropped = im[crop_top_left[0]:crop_bottom_right[0], crop_top_left[1]:crop_bottom_right[1], ...].astype(
            'uint8')

        # make random noise image tensor
        crop_dim = list(c_cropped.shape)
        crop_dim[:2] = size_crop.tolist()
        cropped_image = np.random.randint(256, size=crop_dim).astype('uint8')

        if len(crop_dim) > 2:  # crop 3D or more
            cropped_image[TL_adj[0]:TL_adj[0] + c_cropped.shape[0], TL_adj[1]:TL_adj[1] + c_cropped.shape[1],
            ...] = c_cropped
        else:  # crop 2D
            cropped_image[TL_adj[0]:TL_adj[0] + c_cropped.shape[0],
            TL_adj[1]:TL_adj[1] + c_cropped.shape[1]] = c_cropped

        return cropped_image, crop_top_left2, crop_bottom_right2

    def plot_pole_center(self, video_file, location_stack):
        """

        Parameters
        ----------
        video_file :
            
        location_stack :
            

        Returns
        -------

        """
        video = cv2.VideoCapture(video_file)
        success, frame = video.read()
        location_stack = location_stack + np.array(self.template_image.shape) / 2
        _ = plt.figure()
        plt.imshow(frame)
        plt.scatter(location_stack[:, 0], location_stack[:, 1], s=.2, c='r')
        plt.show()

        threshold = 50  # threshold in pixels
        distance_from_mean = np.sum(np.abs(location_stack - np.mean(location_stack, axis=0)), axis=1)
        exceed_threshold = np.mean(distance_from_mean < threshold) * 100
        print(str(np.round(exceed_threshold, 2)) + '% of the pole locations are within ' + str(
            threshold) + ' pixels from the mean pole location.')

    def track_random(self) -> object:
        """This function will randomly grab a video file within the directory and
        track all frames within there and plot tracked locations over the original image

        Parameters
        ----------

        Returns
        -------

        """
        video_file = np.random.choice(self.video_files, 1)[0]
        print('Testing tracking on ' + video_file)
        img_stack, loc_stack, max_val_stack = self.track(video_file=video_file)
        self.plot_pole_center(video_file=video_file, location_stack=loc_stack)

    def get_trial_and_file_names(self, pos_seps='-_', custom_trial_nums=None, custom_ascii_video_files=None,
                                 print_them=False, num_to_print=None):
        """pos_seps - (default = '-_') will find the number after the last of '-' or '_', can add custom notion for your naming scheme
        custom_trial_nums - override in case this is needed, list of strings equal to length of number of MP4 files (e.g. ['9', '10', '11'])
        custom_ascii_video_files - override in case this is needed, list of full file names (without base directory) must be ASCII format for saving in H5 file later
        to convert LIST (list of strings) to ASCII us -> ASCII_LIST = [os.path.basename(n).encode("ascii", "ignore") for n in LIST]
        print_them - just used to print the names to make sure the naming is what you want before spending all the time tracking.
        num_to_print - if print_them == True, this will print the first N number of files (used in case you are doing a huge amount
        of videos from many directories, we dont want to overload the terminal window)

        Parameters
        ----------
        pos_seps :
             (Default value = '-_')
        custom_trial_nums :
             (Default value = None)
        custom_ascii_video_files :
             (Default value = None)
        print_them :
             (Default value = False)
        num_to_print :
             (Default value = None)

        Returns
        -------

        """
        print(self.video_files[0])
        trial_sep = [k for k in self.video_files[0] if k in pos_seps][-1]  # '-_' are the possible separators
        self.trial_sep = trial_sep
        save_base_name = os.path.basename(self.video_files[0]).split(self.trial_sep)
        self.save_base_name = '-'.join(save_base_name[:-1])
        if custom_trial_nums is None:
            self.trial_nums = list(
                map(lambda s: re.search("^.*" + trial_sep + "([0-9]+)\.", s).group(1), self.video_files))
        else:
            self.trial_nums = custom_trial_nums

        if custom_ascii_video_files is None:
            self.ascii_video_files = [os.path.basename(n).encode("ascii", "ignore") for n in self.video_files]
        else:
            self.ascii_video_files = custom_ascii_video_files

        if print_them:  # print the files and trial nums to make sure they are sorted and match and what the user expects
            print(*['FILE->' + os.path.basename(k1).split('.')[0] + '  ' + k2 + '<-TRIAL#' for k1, k2 in
                    zip(self.video_files[:num_to_print], self.trial_nums[:num_to_print])], sep='\n')

    def track_all_and_save(self, verbose=False, save_directory=None):
        """

        Parameters
        ----------
        verbose :
             (Default value = False)
        save_directory :
             (Default value = None)

        Returns
        -------

        """
        if save_directory is None:
            save_directory = self.video_directory
        save_directory = save_directory + os.path.sep

        """
        This is the major output function of the PoleTracking class. This functions will 
        track all video files within the directory and save an H5 file with the meta information:
        - file_name_nums : this is the trial number extracted from the video file name
        - image: the tracked image stack
        - labels : this is pre-allocation for the CNN to label the images
        - trial_nums_and_frame_nums: 2 dimensional vector. Row 1 = trial number and Row 2 = frame number in that trial
        - in_range: this is pre-allocation for the pole in range tracker
        """
        loc_stack_all = np.asarray([])
        max_val_stack_all = np.asarray([])
        # get the video names and trial numbers -- this is a sperate function
        # so that you can run it before you process the videos to see if the
        # naming and numbers are correct. can run PT.get_trial_and_file_names(True)
        # to print out the file lists before you run everyhting to check.
        if 'ascii_video_files' not in [k for k in dir(self) if '_' not in k[0]]:
            self.get_trial_and_file_names()
        # save data in H5 with name similar to video
        file_name = self.save_base_name + '.h5'
        # create image stack
        final_stack = []
        len_all = 0
        start = time.time()
        h5creator = h5_iterative_creator(save_directory + file_name,
                                         overwrite_if_file_exists=True,
                                         max_img_height=self.template_image.shape[0],
                                         max_img_width=self.template_image.shape[1],
                                         close_and_open_on_each_iteration=False)
        print("Don't worry it will close automatically once finished ")
        for video in tqdm(self.video_files):
            if verbose: print('Tracking... ' + video)
            img_stack, loc_stack, max_val_stack = self.track(video_file=video)
            neg_1_filler_labels = np.float64(np.ones(img_stack.shape[0]) * -1)
            h5creator.add_to_h5(img_stack, neg_1_filler_labels)
            # loc_stack_all.append(loc_stack)
            loc_stack_all = np.vstack((loc_stack_all, loc_stack)) if len(loc_stack_all) != 0 else loc_stack
            # max_val_stack_all.append(max_val_stack)
            max_val_stack = np.asarray(max_val_stack)
            max_val_stack_all = np.hstack((max_val_stack_all, max_val_stack)) if len(
                max_val_stack_all) != 0 else max_val_stack
            len_all += img_stack.shape[0]
        h5creator.close_h5()

        #  frame numbers
        frame_nums = list(map(lambda s: cv2.VideoCapture(s).get(7), self.video_files))

        tnf = np.vstack([np.array(list(self.trial_nums)).astype(int),
                         np.array(list(frame_nums))])

        # populating which trial each frame is in
        fnn = []
        for a, b in zip(list(self.trial_nums), list(frame_nums)):
            fnn = np.concatenate([fnn, np.repeat(int(a), b)])

        # populating whether pole "in_range" with nan values
        in_range = np.empty(len_all)
        in_range[:] = np.nan

        # check to make sure sizes across file names and images are equal
        assert len(fnn) == len_all, '''the fnn and length of frames don't match '''

        with h5py.File(save_directory + file_name, 'r+') as hf:  # with -> auto close in case of failure
            hf.create_dataset('locations_x_y', data=loc_stack_all)
            hf.create_dataset('max_val_stack', data=max_val_stack_all)
            hf.create_dataset('file_name_nums', data=fnn)
            hf.create_dataset('trial_nums_and_frame_nums', data=tnf)
            hf.create_dataset('in_range', data=in_range)
            hf.create_dataset('full_file_names', data=self.ascii_video_files)
            hf.create_dataset('template_img', data=self.template_image)
            hf.create_dataset('frame_nums', data=tnf[1, :])
            hf.close()

        print('H5 file saving under the name ' + file_name)
        print('and placed in ' + save_directory)
        elapsed = time.time() - start
        print('Tracker runtime : ' + str(elapsed / 60) + ' mins')
        return save_directory + file_name

    def track(self, video_file, match_method='cv2.TM_CCOEFF'):
        """this function scans a template image across each frame of the video to identify the pole location.
        This assumes there is a pole at each frame. Cropping optimizes scanning by ~80% and uses the first frame
        as a point of reference.

        Parameters
        ----------
        video_file :

        match_method :
             (Default value = 'cv2.TM_CCOEFF')

        Returns
        -------

        """

        # width and height of img_stacks will be that of template (61x61)
        w, h = self.template_image.shape[::-1]
        max_match_val = []
        # open video at directory
        video = cv2.VideoCapture(video_file)
        if (video.isOpened() == False):
            print('error opening video file')

        fno = 0
        img_list = []
        loc_list = []
        success, og_frame = video.read()
        method = eval(match_method)
        crop_top_left = 0
        pole_center = 0
        tmp1 = 0
        while success:
            # preprocess image
            tmp1 = tmp1 + 1
            if 'frame' in locals() and self.use_narrow_search_to_speed_up:
                frame, crop_top_left, crop_bottom_right = self.crop_image_from_top_left(og_frame,
                                                                                        crop_top_left2,
                                                                                        [w, h],
                                                                                        3)
            else:
                frame = og_frame
            img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY).astype('uint8').copy()

            # Apply template Matching
            res = cv2.matchTemplate(img, self.template_image, method)
            min_val, max_val, min_loc, top_left = cv2.minMaxLoc(res)
            max_match_val.append(max_val)
            top_left = np.flip(np.asarray(top_left))

            # crop image and store
            crop_img, crop_top_left2, crop_bottom_right2 = self.crop_image_from_top_left(og_frame,
                                                                                         top_left + crop_top_left,
                                                                                         [w, h])
            img_list.append(crop_img)
            loc_list.append(np.flip(crop_top_left2))

            # iterate to next frame and crop using current details
            fno += 1
            success, og_frame = video.read()

        img_stack = np.array(img_list, dtype=np.uint8)
        loc_stack = np.array(loc_list)
        return img_stack, loc_stack, max_match_val

    def set_pole_template(self, cust_template):
        """custom template: input must be the 2D matrix (no color channels) and dtype=uint8

        Parameters
        ----------
        cust_template :
            

        Returns
        -------

        """
        self.template_image = cust_template

    def save_template_img(self, cust_save_dir='', cust_save_name='template_img'):
        """save current template image to use for similar session

        Parameters
        ----------
        cust_save_dir :
             (Default value = '')
        cust_save_name :
             (Default value = 'template_img')

        Returns
        -------

        """
        img = Image.fromarray(self.template_image, 'L')
        full_save_name = cust_save_dir + os.path.sep + cust_save_name + '.png'
        img.save(full_save_name)
        print('SAVED... ' + full_save_name)

    def load_template_img(self, img_to_load):
        """

        Parameters
        ----------
        img_to_load :
            

        Returns
        -------

        """
        self.template_image = np.asarray(Image.open(img_to_load))

    def cut_out_pole_template(self, crop_size=[61, 61], frame_num=2000, file_ind=None):
        """

        Parameters
        ----------

        crop_size :
             (Default value = [61)
        61] :
            
        frame_num :
             (Default value = 2000)
        file_ind :
             (Default value = None)

        Returns
        -------

        """
        top_left = str(crop_size[0]) + ' , ' + str(crop_size[1])
        crop_size = np.asarray(crop_size)
        if file_ind is None:
            video_file = np.random.choice(self.video_files, 1)[0]
        else:
            video_file = self.video_files[file_ind]
        plt.ion()
        plt.show()
        change_trig = True
        while 'exit' not in str(top_left).lower():
            if 'skip' in str(top_left).lower():
                video_file = np.random.choice(self.video_files, 1)[0]
                print('Randomly selecting a new video...')
            try:
                top_left = eval('[' + top_left + ']')
                if change_trig:
                    top_left = top_left - np.floor(crop_size / 2).astype('int')
                else:
                    change_trig = True
                print('Sampling video file ' + video_file.split('/')[-1])
                video = cv2.VideoCapture(video_file)
                tmp_fig = plt.figure(figsize=[6, 6])
                video.set(1, frame_num)
                _, frame = video.read()

                plt.imshow(frame)

                tmp_img, a, b = self.crop_image_from_top_left(frame, top_left, crop_size)
                template_image = tmp_img.copy()
                crop_size_2 = np.floor(crop_size / 2).astype('int')
                tmp_img[crop_size_2[0], :] = 0
                tmp_img[:, crop_size_2[1]] = 0

                _ = plt.subplot(122)
                _ = plt.imshow(tmp_img, extent=[top_left[1], tmp_img.shape[1] + top_left[1],
                                                tmp_img.shape[0] + top_left[0], top_left[0]])
                ax = plt.gca()
                ax.set_yticks(np.arange(top_left[0] + .5, top_left[0] + tmp_img.shape[0], 5), minor=True)
                ax.set_xticks(np.arange(top_left[1] + .5, top_left[1] + tmp_img.shape[1], 5), minor=True)

                plt.grid(which='minor', color='red', linestyle='-', linewidth=.2)
                frame2 = frame.copy()
                try:
                    frame2[a[0], :] = 0
                    frame2[b[0], :] = 0
                    frame2[:, a[1]] = 0
                    frame2[:, b[1]] = 0
                except:
                    print('partially out of range not plotting crosshairs on zoomed out image')
                _ = plt.subplot(121)
                _ = plt.imshow(frame2)
                ax = plt.gca()
                ax.set_yticks(np.arange(1, frame2.shape[0], 50), minor=True)
                ax.set_xticks(np.arange(1, frame2.shape[1], 50), minor=True)
                plt.grid(which='minor', color='red', linestyle='-', linewidth=.2)

                tmptop_left = top_left.copy()
                U_in_2_print = top_left + np.floor(crop_size / 2).astype('int')
                plt.draw()
                plt.pause(0.01)
                plt.show()
                top_left = input('Input the pole center coordinates seperated by a comma \ny (0 = top, max = bottom) '
                                 '\nx (0 = left, max = right) \nor enter "exit" to finalize image selection '
                                 '\nor enter "exit all" to terminate the while loop'
                                 '\nor enter "skip" to choose another random video' + '\nlast selection --> '
                                 + str(U_in_2_print[0]) + ' , ' + str(
                    U_in_2_print[1]) + '\n______________________________________________________\n')
                plt.close(tmp_fig)
                if 'exit all' == str(top_left).lower():
                    assert False, 'user exited the program'
            except:
                if 'exit all' == str(top_left).lower():
                    assert False, 'user exited the program'
                if str(top_left).lower() != 'skip':
                    print(
                        'oops try again enter a pair of numbers or enter exit to finish select... or maybe you entered an out of range pair of numbers')
                top_left = tmptop_left.copy()
                top_left = str(top_left[0]) + ' , ' + str(top_left[1])
                change_trig = False

        self.template_image = template_image[:, :, 0]




"""
things to do

fix the update plotting for the regular (not Ipython) python

allow for either saving all of the weights

allow for plotting test set every X
OOOORRRRRR
plot X% of the test set... nah this is not as useful

allow for plotting different plot using subplot I guess is the easiest way
"""
from whacc import utils

 # setting up plotting when usinsave_and_plot_historyg google colab or notebook (though only tested in colab)
if utils.isnotebook():

    from IPython import get_ipython

    ipython = get_ipython()
    ipython.magic("matplotlib inline")
from IPython import display

import copy
from whacc import image_tools
import tensorflow as tf
from tensorflow.keras.callbacks import ModelCheckpoint
from tensorflow import keras
from sklearn.utils import class_weight
import numpy as np
import time
import os
import matplotlib.pyplot as plt
import json


def foo_make_model(model_2_load, dropout=None):
    """
  test if the trainable setting stay to what they are for output model
  """
    model2 = tf.keras.models.load_model(model_2_load)
    if dropout is not None:
        feature_batch = model2.layers[0].output
        dropout_layer = tf.keras.layers.Dropout(dropout, input_shape=model2.layers[0].input.shape)
        # Model Stacking
        model = tf.keras.Sequential([
            model2.layers[0],
            model2.layers[1],
            dropout_layer,
            model2.layers[2]
        ])
        model.layers[0].trainable = False  # base
        model.layers[1].trainable = False  #
        model.layers[2].trainable = False  # dropout layer
        model.layers[3].trainable = True  # class head

    else:

        model = tf.keras.Sequential([
            model2.layers[0],
            model2.layers[1],
            model2.layers[2]
        ])
        model.layers[0].trainable = False  # base
        model.layers[1].trainable = False  #
        model.layers[2].trainable = True  # class head
    METRICS = [
        keras.metrics.AUC(name='auc'),
        keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
        keras.metrics.TruePositives(name='tp'),
        keras.metrics.FalsePositives(name='fp'),
        keras.metrics.TrueNegatives(name='tn'),
        keras.metrics.FalseNegatives(name='fn'),
        keras.metrics.Precision(name='precision'),
        keras.metrics.Recall(name='recall')
    ]
    model.compile(optimizer=tf.keras.optimizers.RMSprop(10 ** -6),
                  loss=tf.keras.losses.BinaryCrossentropy(),
                  metrics=METRICS)

    return model


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
def foo_start_running(model_2_load, training_h5_list, validation_h5_list, dropout=None, epochs=100,
                      batch_size=50, learning_rate=10 ** -6, patience=2, monitor='val_loss',
                      key_name='labels', verbose_=1, save_name=None, model_save_dir=None,
                      save_best_only=False, add_callback_list=[]):
    if save_name is not None or model_save_dir is not None:
        assert os.path.isdir(model_save_dir), "model_save_dir must be a directory"
        assert save_name is not None, "if you want to save the model model_save_dir and save_name must not be None"
        MOD_BASE_NAME = model_2_load.split(os.path.sep)[-1].split('.')[0]
        ModelCheckpoint_save_name = model_save_dir + MOD_BASE_NAME + '_' + save_name + '.hdf5'
        add_callback_list.append(ModelCheckpoint(ModelCheckpoint_save_name, save_best_only=save_best_only))

    model = foo_make_model(model_2_load, dropout=dropout)
    # CREATE TRAINING BATCH GENERATOR \/
    my_training_batch_generator = image_tools.ImageBatchGenerator(batch_size, training_h5_list)
    # my_validation_batch_generator
    my_validation_batch_generator = image_tools.ImageBatchGenerator(batch_size, validation_h5_list)

    model.optimizer.learning_rate = learning_rate

    # Class imbalance weighting
    all_y = image_tools.get_h5_key_and_concatenate(training_h5_list, key_name=key_name)
    rebalance = class_weight.compute_class_weight('balanced', [0, 1], all_y.flatten())
    class_weights = {i: rebalance[i] for i in range(2)}

    callbacks = [keras.callbacks.EarlyStopping(monitor=monitor, patience=patience)]
    for k in add_callback_list:
        callbacks.append(k)
    start = time.time()
    history = model.fit(my_training_batch_generator,
                        epochs=epochs,
                        validation_data=my_validation_batch_generator,
                        callbacks=callbacks,
                        class_weight=class_weights,
                        verbose=verbose_)
    run_time = time.time() - start
    print('total run time :' + str(round(run_time / 60)), ' minutes')
    info_dict = {'monitor': callbacks[0].monitor,
                 'patience': callbacks[0].patience,
                 'epochs': epochs,
                 'layers_re_trainable': [k.trainable for k in model.layers] * 1,
                 're_learning_rate': float(model.optimizer.learning_rate),
                 'model_2_load': model_2_load,
                 'training_h5_list': training_h5_list,
                 'validation_h5_list': validation_h5_list,
                 'dropout': dropout,
                 'run_time': run_time}
    return info_dict


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
class save_and_plot_history(keras.callbacks.Callback):
    def __init__(self, my_test_batch_generator, key_name='labels', plot_metric_inds=[0]):
        self.all_y = image_tools.get_h5_key_and_concatenate(my_test_batch_generator.H5_file_list, key_name=key_name)
        self.my_test_batch_generator = my_test_batch_generator
        self.all_logs = np.asarray([])
        self.markers = np.tile(['-', '--', ':'], len(plot_metric_inds))
        c = plt.rcParams["axes.prop_cycle"].by_key()["color"]
        self.all_colors = c
        self.color = np.repeat([c[k] for k in plot_metric_inds], 3)
        self.plot_metric_inds = plot_metric_inds

    def on_epoch_end(self, epoch, logs=None):
        if epoch == 0 or 'logs_names' not in dir(self):

            self.L = len(self.model.metrics_names)
            L = len(self.model.metrics_names)
            self.matching_inds = np.reshape(np.linspace(0, L * 3 - 1, L * 3), [3, -1]).astype('int')
            self.plot_metric_inds = self.plot_metric_inds + [k + L for k in self.plot_metric_inds] + [k + L * 2 for k in
                                                                                                      self.plot_metric_inds]
            self.plot_metric_inds = np.swapaxes(np.reshape(self.plot_metric_inds, [3, -1]), 1, 0).flatten()
            self.fig = plt.figure(self.plot_metric_inds[0], figsize=[15, 10])
            self.logs_names = self.model.metrics_names + [k + '_val' for k in self.model.metrics_names] + [k + '_test'
                                                                                                           for k in
                                                                                                           self.model.metrics_names]
            self.logs_names_selected = []
            for k in self.plot_metric_inds:
                self.logs_names_selected.append(self.logs_names[k])
        if logs is not None:
            log_list = []
            _ = [log_list.append(logs[k]) for k in logs.keys()]
            logs_test = self.model.evaluate(self.my_test_batch_generator)
            log_list = log_list + logs_test
            if len(self.all_logs) != 0:
                self.all_logs = np.vstack((self.all_logs, np.asarray(log_list)))
            else:
                self.all_logs = np.concatenate((self.all_logs, np.asarray(log_list)), axis=0)
            # if epoch > 0:
            if len(self.all_logs.shape) > 1:
                display.clear_output(wait=True)
                for kmark, kcol, k in zip(self.markers, self.color, self.plot_metric_inds):
                    plt.plot(self.all_logs[:, k], color=kcol, linestyle=kmark)

                plt.legend(self.logs_names_selected)
                plt.grid(True)
                display.display(plt.gcf())


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
#  for auto saving get the next number for saving in order
def foo_get_next_save_num(dir_str):
    num_names = [0]
    for k in utils.get_files(dir_str, '*mod_test_*'):
        try:
            num_names.append(int(k.split('.png')[-2][-3:]))
        except:
            pass
        try:
            num_names.append(int(k.split('.json')[-2][-3:]))
        except:
            pass
    save_num = str(np.max(num_names) + 1).zfill(4)
    return save_num


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
# text for figure
def make_text_for_fig(training_info):
    tmp_str = ''
    for k in training_info.keys():
        if k in ['monitor', 'patience', 'epochs', 'layers_re_trainable', 're_learning_rate', 'model_2_load',
                 'training_h5_list', 'validation_h5_list', 'dropout', 'run_time']:
            a = copy.deepcopy(training_info[k])
            if k in ['model_2_load']:
                a = "\n  " + a.split(os.path.sep)[-1]
            elif k in ['training_h5_list', 'validation_h5_list']:
                for i, kk in enumerate(a):
                    a[i] = "\n  " + str(i + 1) + ') ' + kk.split(os.path.sep)[-1]
                a = ''.join(a)
            tmp_str = tmp_str + k + ' : ' + str(a) + '\n'
        tmp2 = tmp_str.split('\\n')
        tmp_str = '\n'.join(tmp2)
    return tmp_str


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
# save the info file
def pack_training_info(training_info, save_and_plot_history_var):
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    for k in training_info.keys():
        if 'numpy.ndarray' in str(type(training_info[k])):
            training_info[k] = training_info[k].tolist()
    return training_info


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########
def plot_train_hist(training_info, all_inds, ylims=[None, None], text_str=None):
    _ = plt.figure(figsize=[15, 8])
    _ = plt.gcf().text(.95, .3, text_str, fontsize=20)
    c = np.tile(training_info['all_colors'][:training_info['L']], 3)
    leg_app = []
    for ind in all_inds:
        for k, kline in zip(np.asarray(training_info['matching_inds'])[:, ind],
                            np.tile(training_info['markers'][:3], 20)):
            x = np.asarray(training_info['all_logs'])[:, k]
            _ = plt.plot(x, linestyle=kline, color=c[k])
            leg_app.append(training_info['logs_names'][k])
        _ = plt.legend(leg_app)
    plt.ylim(ylims)


###########$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###########

def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = make_text_for_fig(training_info)
    training_info = pack_training_info(training_info, save_and_plot_history_1)
    save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    plot_train_hist(training_info, [1, 2], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC_AUC_' + save_num + '.png', bbox_inches="tight")
    plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS_' + save_num + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'mod_test_data_' + save_num + '.json', 'w') as f:
        json.dump(training_info, f)

def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = transfer_learning.make_text_for_fig(training_info)
    training_info = transfer_learning.pack_training_info(training_info, save_and_plot_history_1)
    # save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    transfer_learning.plot_train_hist(training_info, [1], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC' + '.png', bbox_inches="tight")
    transfer_learning.plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS' + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'model_eval_each_epoch' + '.json', 'w') as f:
        json.dump(training_info, f)
